@@ -12,22 +12,26 @@ static int
 chunli_draw(struct notcurses* nc, const char* ext, int count, const nccell* b){
   chunli chuns[CHUNS];
   char file[20];
-  int dimx, dimy;
+  unsigned dimx, dimy;
+  ncplane_dim_yx(notcurses_stdplane_const(nc), &dimy, &dimx);
   struct timespec iterdelay;
   timespec_div(&demodelay, 10, &iterdelay);
   for(int i = 0 ; i < count ; ++i){
-    notcurses_refresh(nc, &dimy, &dimx);
     snprintf(file, sizeof(file), "chunli%d.%s", i + 1, ext);
     chuns[i].path = find_data(file);
     chuns[i].ncv = ncvisual_from_file(chuns[i].path);
     if(chuns[i].ncv == NULL){
       return -1;
     }
-    if((chuns[i].n = ncvisual_render(nc, chuns[i].ncv, NULL)) == NULL){
+    struct ncvisual_options vopts = {
+      .n = notcurses_stdplane(nc),
+      .flags = NCVISUAL_OPTION_CHILDPLANE,
+    };
+    if((chuns[i].n = ncvisual_blit(nc, chuns[i].ncv, &vopts)) == NULL){
       return -1;
     }
     ncplane_set_base_cell(chuns[i].n, b);
-    int thisx, thisy;
+    unsigned thisx, thisy;
     ncplane_dim_yx(chuns[i].n, &thisy, &thisx);
     if(ncplane_move_yx(chuns[i].n, (dimy - thisy) / 2, (dimx - thisx) / 2)){
       return -1;
@@ -43,17 +47,19 @@ chunli_draw(struct notcurses* nc, const char* ext, int count, const nccell* b){
 }
 
 // test of sprites from files
-int chunli_demo(struct notcurses* nc){
+int chunli_demo(struct notcurses* nc, uint64_t startns){
+  (void)startns;
   if(!notcurses_canopen_images(nc)){
     return 0;
   }
   struct timespec iterdelay;
   timespec_div(&demodelay, 10, &iterdelay);
-  int ret, dimx, dimy;
-  notcurses_refresh(nc, &dimy, &dimx);
-  nccell b = CELL_TRIVIAL_INITIALIZER;
-  cell_set_fg_alpha(&b, CELL_ALPHA_TRANSPARENT);
-  cell_set_bg_alpha(&b, CELL_ALPHA_TRANSPARENT);
+  int ret;
+  unsigned dimx, dimy;
+  ncplane_dim_yx(notcurses_stdplane_const(nc), &dimy, &dimx);
+  nccell b = NCCELL_TRIVIAL_INITIALIZER;
+  nccell_set_fg_alpha(&b, NCALPHA_TRANSPARENT);
+  nccell_set_bg_alpha(&b, NCALPHA_TRANSPARENT);
   if( (ret = chunli_draw(nc, "bmp", CHUNS, &b)) ){
     return ret;
   }
@@ -68,15 +74,18 @@ int chunli_demo(struct notcurses* nc){
     }
     free(path);
     struct ncplane* ncp;
-    if((ncp = ncvisual_render(nc, ncv, NULL)) == NULL){
+    struct ncvisual_options vopts = {
+      .x = NCALIGN_CENTER,
+      .y = NCALIGN_CENTER,
+      .n = notcurses_stdplane(nc),
+      .flags = NCVISUAL_OPTION_CHILDPLANE |
+               NCVISUAL_OPTION_HORALIGNED |
+               NCVISUAL_OPTION_VERALIGNED,
+    };
+    if((ncp = ncvisual_blit(nc, ncv, &vopts)) == NULL){
       return -1;
     }
     ncplane_set_base_cell(ncp, &b);
-    int thisx, thisy;
-    ncplane_dim_yx(ncp, &thisy, &thisx);
-    if(ncplane_move_yx(ncp, (dimy - thisy) / 2, (dimx - thisx) / 2)){
-      return -1;
-    }
     DEMO_RENDER(nc);
     demo_nanosleep(nc, &iterdelay);
     ncvisual_destroy(ncv);
