@@ -25,31 +25,31 @@ extern atomic_bool interrupted;
 // heap-allocated, caller must free. locates data files per command line args.
 char* find_data(const char* datum);
 
-int animate_demo(struct notcurses* nc);
-int unicodeblocks_demo(struct notcurses* nc);
-int witherworm_demo(struct notcurses* nc);
-int box_demo(struct notcurses* nc);
-int trans_demo(struct notcurses* nc);
-int chunli_demo(struct notcurses* nc);
-int dragon_demo(struct notcurses* nc);
-int qrcode_demo(struct notcurses* nc);
-int grid_demo(struct notcurses* nc);
-int fission_demo(struct notcurses* nc);
-int highcontrast_demo(struct notcurses* nc);
-int jungle_demo(struct notcurses* nc);
-int yield_demo(struct notcurses* nc);
-int mojibake_demo(struct notcurses* nc);
-int normal_demo(struct notcurses* nc);
-int sliding_puzzle_demo(struct notcurses* nc);
-int view_demo(struct notcurses* nc);
-int eagle_demo(struct notcurses* nc);
-int reel_demo(struct notcurses* nc);
-int xray_demo(struct notcurses* nc);
-int keller_demo(struct notcurses* nc);
-int luigi_demo(struct notcurses* nc);
-int zoo_demo(struct notcurses* nc);
-int intro(struct notcurses* nc);
-int outro(struct notcurses* nc);
+int animate_demo(struct notcurses* nc, uint64_t startns);
+int uniblock_demo(struct notcurses* nc, uint64_t startns);
+int whiteout_demo(struct notcurses* nc, uint64_t startns);
+int box_demo(struct notcurses* nc, uint64_t startns);
+int trans_demo(struct notcurses* nc, uint64_t startns);
+int chunli_demo(struct notcurses* nc, uint64_t startns);
+int dragon_demo(struct notcurses* nc, uint64_t startns);
+int qrcode_demo(struct notcurses* nc, uint64_t startns);
+int grid_demo(struct notcurses* nc, uint64_t startns);
+int fission_demo(struct notcurses* nc, uint64_t startns);
+int highcon_demo(struct notcurses* nc, uint64_t startns);
+int jungle_demo(struct notcurses* nc, uint64_t startns);
+int yield_demo(struct notcurses* nc, uint64_t startns);
+int mojibake_demo(struct notcurses* nc, uint64_t startns);
+int normal_demo(struct notcurses* nc, uint64_t startns);
+int sliders_demo(struct notcurses* nc, uint64_t startns);
+int view_demo(struct notcurses* nc, uint64_t startns);
+int eagle_demo(struct notcurses* nc, uint64_t startns);
+int reel_demo(struct notcurses* nc, uint64_t startns);
+int xray_demo(struct notcurses* nc, uint64_t startns);
+int keller_demo(struct notcurses* nc, uint64_t startns);
+int luigi_demo(struct notcurses* nc, uint64_t startns);
+int zoo_demo(struct notcurses* nc, uint64_t startns);
+int intro_demo(struct notcurses* nc, uint64_t startns);
+int outro_demo(struct notcurses* nc, uint64_t startns);
 
 /*------------------------------- demo input API --------------------------*/
 int input_dispatcher(struct notcurses* nc);
@@ -140,7 +140,7 @@ struct ncmenu* menu_create(struct notcurses* nc);
 int hud_destroy(void);
 
 // let the HUD know about an upcoming demo
-int hud_schedule(const char* demoname);
+int hud_schedule(const char* demoname, uint64_t startns);
 
 // demos should not call notcurses_render() themselves, but instead call
 // demo_render(), which will ensure the HUD stays on the top of the z-stack.
@@ -213,7 +213,7 @@ pulser(struct notcurses* nc, struct ncplane* ncp __attribute__ ((unused)),
   struct timespec* start = curry;
   struct timespec now;
   clock_gettime(CLOCK_MONOTONIC, &now);
-  if(timespec_to_ns(&now) - timespec_to_ns(start) >= timespec_to_ns(&demodelay) * 10 / 3){
+  if(timespec_to_ns(&now) - timespec_to_ns(start) >= timespec_to_ns(&demodelay)){
     return 2;
   }
   return demo_render(nc);
