@@ -10,23 +10,23 @@ typedef struct nciqueue {
 } nciqueue;
 
 // a pipe on which we write upon receipt of input, so that demos
-// can reliably multiplex against other fds. freebsd doesn't have
-// eventfd, alas.
+// can reliably multiplex against other fds. osx doesn't have
+// eventfd, alas (freebsd added it in 13.0).
 static int input_pipefds[2] = {-1, -1};
 
 static pthread_t tid;
 static nciqueue* queue;
 static nciqueue** enqueue = &queue;
-static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
 static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t cond; // use pthread_condmonotonic_init()
 
 static int
 handle_mouse(const ncinput* ni){
-  if(ni->id != NCKEY_BUTTON1 && ni->id != NCKEY_RELEASE){
+  if(ni->id != NCKEY_BUTTON1){
     return 0;
   }
   int ret;
-  if(ni->id == NCKEY_RELEASE){
+  if(ni->evtype == NCTYPE_RELEASE){
     ret = hud_release();
     if(ret < 0){
       ret = fpsplot_release();
@@ -47,10 +47,8 @@ handle_mouse(const ncinput* ni){
 // absolute deadline, so convert it up.
 uint32_t demo_getc(struct notcurses* nc, const struct timespec* ts, ncinput* ni){
   struct timespec now;
-  clock_gettime(CLOCK_REALTIME, &now);
+  clock_gettime(CLOCK_MONOTONIC, &now);
   uint64_t ns;
-  // yes, i'd like CLOCK_MONOTONIC too, but pthread_cond_timedwait() is based off
-  // of crappy CLOCK_REALTIME :/
   // abstime shouldn't be further out than our maximum sleep time -- this can
   // lead to 0 frames output during the wait
   if(ts){
@@ -68,7 +66,7 @@ uint32_t demo_getc(struct notcurses* nc, const struct timespec* ts, ncinput* ni)
   do{
     pthread_mutex_lock(&lock);
     while(!queue){
-      clock_gettime(CLOCK_REALTIME, &now);
+      clock_gettime(CLOCK_MONOTONIC, &now);
       if(timespec_to_ns(&now) > timespec_to_ns(&abstime)){
         pthread_mutex_unlock(&lock);
         return 0;
@@ -86,18 +84,10 @@ uint32_t demo_getc(struct notcurses* nc, const struct timespec* ts, ncinput* ni)
     if(!menu_or_hud_key(nc, &q->ni)){
       if(nckey_mouse_p(q->ni.id)){
         if(!handle_mouse(&q->ni)){
-          if(id == 'L' && q->ni.ctrl){
-            notcurses_refresh(nc, NULL, NULL);
-          }else{
-            handoff = true;
-          }
-        }
-      }else{
-        if(id == 'L' && q->ni.ctrl){
-          notcurses_refresh(nc, NULL, NULL);
-        }else{
           handoff = true;
         }
+      }else{
+        handoff = true;
       }
     }
     if(handoff && ni){
@@ -119,7 +109,7 @@ pass_along(const ncinput* ni){
   pthread_mutex_unlock(&lock);
   const uint64_t eventcount = 1;
   int ret = 0;
-  if(write(input_pipefds[0], &eventcount, sizeof(eventcount)) < 0){
+  if(write(input_pipefds[1], &eventcount, sizeof(eventcount)) < 0){
     ret = -1;
   }
   pthread_cond_signal(&cond);
@@ -131,10 +121,13 @@ ultramegaok_demo(void* vnc){
   ncinput ni;
   struct notcurses* nc = vnc;
   uint32_t id;
-  while((id = notcurses_getc_blocking(nc, &ni)) != (uint32_t)-1){
+  while((id = notcurses_get_blocking(nc, &ni)) != (uint32_t)-1){
     if(id == 0){
       continue;
     }
+    if(id == NCKEY_EOF){
+      break;
+    }
     // go ahead and pass keyboard through to demo, even if it was a 'q' (this
     // might cause the demo to exit immediately, as is desired). we can't just
     // mess with the menu/HUD in our own context, as the demo thread(s) might
@@ -155,8 +148,14 @@ int input_dispatcher(struct notcurses* nc){
   if(input_pipefds[0] >= 0){
     return -1;
   }
+  if(pthread_condmonotonic_init(&cond)){
+    fprintf(stderr, "error creating monotonic condvar\n");
+    return -1;
+  }
   // freebsd doesn't have eventfd :/ and apple doesn't even have pipe2() =[ =[
-#ifdef __APPLE__
+  // omg windows doesn't have pipe() fml FIXME
+#ifndef __MINGW32__
+#if defined(__APPLE__)
   if(pipe(input_pipefds)){
 #else
   if(pipe2(input_pipefds, O_CLOEXEC | O_NONBLOCK)){
@@ -164,6 +163,7 @@ int input_dispatcher(struct notcurses* nc){
     fprintf(stderr, "Error creating pipe (%s)\n", strerror(errno));
     return -1;
   }
+#endif
   if(pthread_create(&tid, NULL, ultramegaok_demo, nc)){
     close(input_pipefds[0]);
     close(input_pipefds[1]);
@@ -181,6 +181,7 @@ int stop_input(void){
     ret |= close(input_pipefds[0]);
     ret |= close(input_pipefds[1]);
     input_pipefds[0] = input_pipefds[1] = -1;
+    ret |= pthread_cond_destroy(&cond);
   }
   return ret;
 }
