@@ -400,11 +400,12 @@ bool ncreader_offer_input(ncreader* n, const ncinput* ni){
   }else if(nckey_synthesized_p(ni->id)){
     return false;
   }
-  // FIXME need to collect full EGCs
-  char wbuf[WCHAR_MAX_UTF8BYTES + 1];
-  // FIXME breaks for wint_t < 32bits
-  if(snprintf(wbuf, sizeof(wbuf), "%lc", (wint_t)ni->id) < (int)sizeof(wbuf)){
-    ncreader_write_egc(n, wbuf);
+
+  for (int c=0; ni->eff_text[c]!=0; c++){
+    unsigned char egc[5]={0};
+    if(notcurses_ucs32_to_utf8(&ni->eff_text[c], 1, egc, 4)>=0){
+      ncreader_write_egc(n, (char*)egc);
+    }
   }
   return true;
 }
