@@ -114,8 +114,7 @@ typedef enum {
 
 static int
 draw_borders(ncplane* n, unsigned mask, uint64_t channel, direction_e direction){
-  int lenx, leny;
-  int ret = 0;
+  unsigned lenx, leny;
   ncplane_dim_yx(n, &leny, &lenx);
   int maxx = lenx - 1;
   int maxy = leny - 1;
@@ -142,6 +141,7 @@ draw_borders(ncplane* n, unsigned mask, uint64_t channel, direction_e direction)
   // we're left following the previous stanza, end based on maxhorizy.
   const bool candrawbottom = y <= maxy || direction == DIRECTION_UP || (mask & NCBOXMASK_TOP);
   const int maxhorizy = maxy - (candrawbottom && !(mask & NCBOXMASK_BOTTOM));
+  int ret = 0;
   while(y <= maxhorizy){
     if(!(mask & NCBOXMASK_LEFT)){
       ret |= ncplane_cursor_move_yx(n, y, 0);
@@ -186,7 +186,10 @@ tablet_geom(const ncreel* nr, nctablet* t, int* begx, int* begy,
 //fprintf(stderr, "jigsawing %p with %d/%d dir %d\n", t, frontiertop, frontierbottom, direction);
   *begy = 0;
   *begx = 0;
-  ncplane_dim_yx(nr->p, leny, lenx);
+  unsigned uleny, ulenx;
+  ncplane_dim_yx(nr->p, &uleny, &ulenx);
+  *leny = uleny;
+  *lenx = ulenx;
   if(frontiertop < 0){
     if(direction == DIRECTION_UP){
       return -1;
@@ -231,6 +234,74 @@ tablet_geom(const ncreel* nr, nctablet* t, int* begx, int* begy,
   return 0;
 }
 
+// kill the planes associated with |t|
+static void
+nctablet_wipeout(nctablet* t){
+  if(t){
+    if(ncplane_set_widget(t->p, NULL, NULL) == 0){
+      ncplane_family_destroy(t->p);
+    }
+    t->p = NULL;
+    t->cbp = NULL;
+  }
+}
+
+// destroy all existing tablet planes pursuant to redraw
+static void
+clean_reel(ncreel* r){
+  nctablet* vft = r->vft;
+  if(vft){
+    for(nctablet* n = vft->next ; n->p && n != vft ; n = n->next){
+//fprintf(stderr, "CLEANING NEXT: %p (%p)\n", n, n->p);
+      nctablet_wipeout(n);
+    }
+    for(nctablet* n = vft->prev ; n->p && n != vft ; n = n->prev){
+//fprintf(stderr, "CLEANING PREV: %p (%p)\n", n, n->p);
+      nctablet_wipeout(n);
+    }
+//fprintf(stderr, "CLEANING VFT: %p (%p)\n", vft, vft->p);
+    nctablet_wipeout(vft);
+    r->vft = NULL;
+  }
+}
+
+// used as ncplane widget destructor callback, and likewise the heart of
+// ncreel_del(). without this, at context collapse time we'd wipe out all the
+// planes (without wiping out their subsidiaries), and ncreel_destroy() would
+// walk freed memory. as we have no handle on the ncreel, we do not update its
+// metadata (tabletcount), nor redraw, but who cares? we're shutting down.
+static void
+nctablet_delete_internal(struct nctablet* t){
+  t->prev->next = t->next;
+  t->next->prev = t->prev;
+  if(t->p){
+    if(ncplane_set_widget(t->p, NULL, NULL) == 0){
+      ncplane_family_destroy(t->p);
+    }
+  }
+  free(t);
+}
+
+int ncreel_del(ncreel* nr, struct nctablet* t){
+  if(t == NULL){
+    return -1;
+  }
+  if(nr->tablets == t){
+    if((nr->tablets = t->next) == t){
+      nr->tablets = NULL;
+    }
+    // FIXME ought set direction based on actual location of replacement tablet
+    nr->direction = LASTDIRECTION_DOWN;
+  }
+  if(nr->vft == t){
+    clean_reel(nr); // maybe just make nr->tablets the vft?
+  }
+  nctablet_delete_internal(t);
+  --nr->tabletcount;
+  ncreel_redraw(nr);
+  return 0;
+}
+
 // Draw the specified tablet, if possible. DIRECTION_UP means we're laying out
 // bottom-to-top. DIRECTION_DOWN means top-to-bottom. 'frontier{top, bottom}'
 // are the lines to which we'll be fitting the tablet ('frontiertop' to our
@@ -263,6 +334,7 @@ ncreel_draw_tablet(const ncreel* nr, nctablet* t, int frontiertop,
 //fprintf(stderr, "failure creating border plane %d %d %d %d\n", leny, lenx, begy, begx);
     return -1;
   }
+  ncplane_set_widget(t->p, t, (void(*)(void*))nctablet_delete_internal);
   // we allow the callback to use a bound plane that lives above our border
   // plane, thus preventing the callback from spilling over the tablet border.
   int cby = 0, cbx = 0, cbleny = leny, cblenx = lenx;
@@ -298,7 +370,7 @@ ncreel_draw_tablet(const ncreel* nr, nctablet* t, int frontiertop,
     int ll = t->cbfxn(t, direction == DIRECTION_DOWN);
 //fprintf(stderr, "RETURNRETURNRETURN %p %d (%d, %d, %d) DIR %d\n", t, ll, cby, cbleny, leny, direction);
     if(ll > cbleny){
-      logwarn("Tablet callback returned %d lines, %d allowed\n", ll, cbleny);
+      logwarn("tablet callback returned %d lines, %d allowed", ll, cbleny);
       ll = cbleny;
     }
     if(ll != cbleny){
@@ -307,7 +379,7 @@ ncreel_draw_tablet(const ncreel* nr, nctablet* t, int frontiertop,
       if(ll){ // must be smaller than the space we provided; add back bottom
         ncplane_resize_simple(t->cbp, ll, cblenx);
       }else{
-        ncplane_destroy_family(t->cbp);
+        ncplane_family_destroy(t->cbp);
         t->cbp = NULL;
       }
       // resize the borderplane iff we got smaller
@@ -318,7 +390,7 @@ ncreel_draw_tablet(const ncreel* nr, nctablet* t, int frontiertop,
       }
       // We needn't move the resized plane if drawing down, or the focused plane.
       // The focused tablet will have been resized properly above, but it might
-      // be out of position (the focused tablet ought move as little as possible). 
+      // be out of position (the focused tablet ought move as little as possible).
       // Move it back to the frontier, or the nearest line above if it has grown.
       if(nr->tablets == t){
         if(leny - frontiertop + 1 < ll){
@@ -412,14 +484,14 @@ trim_reel_overhang(ncreel* r, nctablet* top, nctablet* bottom){
   }
 //fprintf(stderr, "trimming: top %p bottom %p\n", top->p, bottom->p);
   ncplane_yx(top->p, &y, NULL);
-  int ylen, xlen;
+  unsigned ylen, xlen;
   ncplane_dim_yx(top->p, &ylen, &xlen);
   const int miny = !(r->ropts.bordermask & NCBOXMASK_TOP);
   int boty = y + ylen - 1;
 //fprintf(stderr, "top: %dx%d @ %d, miny: %d\n", ylen, xlen, y, miny);
   if(boty < miny){
 //fprintf(stderr, "NUKING top!\n");
-    ncplane_destroy_family(top->p);
+    ncplane_family_destroy(top->p);
     top->p = NULL;
     top->cbp = NULL;
     top = top->next;
@@ -427,7 +499,7 @@ trim_reel_overhang(ncreel* r, nctablet* top, nctablet* bottom){
   }else if(y < miny){
     int ynew = ylen - (miny - y);
     if(ynew <= 0){
-      ncplane_destroy_family(top->p);
+      ncplane_family_destroy(top->p);
       top->p = NULL;
       top->cbp = NULL;
     }else{
@@ -436,7 +508,7 @@ trim_reel_overhang(ncreel* r, nctablet* top, nctablet* bottom){
       }
       if(top->cbp){
         if(ynew == !(r->ropts.tabletmask & NCBOXMASK_TOP)){
-          ncplane_destroy_family(top->cbp);
+          ncplane_family_destroy(top->cbp);
           top->cbp = NULL;
         }else{
           ncplane_dim_yx(top->cbp, &ylen, &xlen);
@@ -451,38 +523,42 @@ trim_reel_overhang(ncreel* r, nctablet* top, nctablet* bottom){
       }
     }
   }
-  ncplane_dim_yx(bottom->p, &ylen, &xlen);
-  ncplane_yx(bottom->p, &y, NULL);
-  const int maxy = ncplane_dim_y(r->p) - (1 + !(r->ropts.bordermask & NCBOXMASK_BOTTOM));
-  boty = y + ylen - 1;
-//fprintf(stderr, "bot: %dx%d @ %d, maxy: %d\n", ylen, xlen, y, maxy);
-  if(maxy < y){
-//fprintf(stderr, "NUKING bottom!\n");
-    ncplane_destroy_family(bottom->p);
-    bottom->p = NULL;
-    bottom->cbp = NULL;
-    bottom = bottom->prev;
-    return trim_reel_overhang(r, top, bottom);
-  }if(maxy < boty){
-    int ynew = ylen - (boty - maxy);
-    if(ynew <= 0){
-      ncplane_destroy_family(bottom->p);
+  if(bottom->p){
+    ncplane_dim_yx(bottom->p, &ylen, &xlen);
+    ncplane_yx(bottom->p, &y, NULL);
+    const int maxy = ncplane_dim_y(r->p) - (1 + !(r->ropts.bordermask & NCBOXMASK_BOTTOM));
+    boty = y + ylen - 1;
+  //fprintf(stderr, "bot: %dx%d @ %d, maxy: %d\n", ylen, xlen, y, maxy);
+    if(maxy < y){
+  //fprintf(stderr, "NUKING bottom!\n");
+      if(ncplane_set_widget(bottom->p, NULL, NULL) == 0){
+        ncplane_family_destroy(bottom->p);
+      }
       bottom->p = NULL;
       bottom->cbp = NULL;
-    }else{
-      if(ncplane_resize(bottom->p, 0, 0, ynew, xlen, 0, 0, ynew, xlen)){
-        return -1;
-      }
-//fprintf(stderr, "TRIMMED bottom %p from %d to %d (%d)\n", bottom->p, ylen, ynew, maxy - boty);
-      if(bottom->cbp){
-        if(ynew == !(r->ropts.tabletmask & NCBOXMASK_BOTTOM)){
-          ncplane_destroy_family(bottom->cbp);
-          bottom->cbp = NULL;
-        }else{
-          ncplane_dim_yx(bottom->cbp, &ylen, &xlen);
-          ynew -= !(r->ropts.tabletmask & NCBOXMASK_BOTTOM);
-          if(ncplane_resize(bottom->cbp, 0, 0, ynew, xlen, 0, 0, ynew, xlen)){
-            return -1;
+      bottom = bottom->prev;
+      return trim_reel_overhang(r, top, bottom);
+    }if(maxy < boty){
+      int ynew = ylen - (boty - maxy);
+      if(ynew <= 0){
+        ncplane_family_destroy(bottom->p);
+        bottom->p = NULL;
+        bottom->cbp = NULL;
+      }else{
+        if(ncplane_resize(bottom->p, 0, 0, ynew, xlen, 0, 0, ynew, xlen)){
+          return -1;
+        }
+  //fprintf(stderr, "TRIMMED bottom %p from %d to %d (%d)\n", bottom->p, ylen, ynew, maxy - boty);
+        if(bottom->cbp){
+          if(ynew == !(r->ropts.tabletmask & NCBOXMASK_BOTTOM)){
+            ncplane_family_destroy(bottom->cbp);
+            bottom->cbp = NULL;
+          }else{
+            ncplane_dim_yx(bottom->cbp, &ylen, &xlen);
+            ynew -= !(r->ropts.tabletmask & NCBOXMASK_BOTTOM);
+            if(ncplane_resize(bottom->cbp, 0, 0, ynew, xlen, 0, 0, ynew, xlen)){
+              return -1;
+            }
           }
         }
       }
@@ -499,14 +575,15 @@ tighten_reel_down(ncreel* r, int ybot){
     if(cur->p == NULL){
       break;
     }
-    int cury, curx, ylen;
+    int cury, curx;
+    unsigned ylen;
     ncplane_yx(cur->p, &cury, &curx);
     ncplane_dim_yx(cur->p, &ylen, NULL);
-    if(cury <= ybot - ylen - 1){
+    if(cury <= ybot - (int)ylen - 1){
       break;
     }
 //fprintf(stderr, "tightening %p down to %d from %d\n", cur, ybot - ylen, cury);
-    cury = ybot - ylen;
+    cury = ybot - (int)ylen;
     ncplane_move_yx(cur->p, cury, curx);
     ybot = cury - 1;
     if((cur = cur->prev) == r->tablets){
@@ -557,7 +634,7 @@ tighten_reel(ncreel* r){
         return -1;
       }
     }
-    int ylen;
+    unsigned ylen;
     ncplane_dim_yx(cur->p, &ylen, NULL);
     expected += ylen + 1;
 //fprintf(stderr, "bottom (%p) gets %p\n", bottom, cur);
@@ -570,14 +647,15 @@ tighten_reel(ncreel* r){
   cur = r->tablets;
   if(cur){
     const ncplane* n = cur->p;
-    int yoff, ylen, rylen;
+    int yoff;
+    unsigned ylen, rylen;
     ncplane_dim_yx(r->p, &rylen, NULL);
     ncplane_yx(n, &yoff, NULL);
     ncplane_dim_yx(n, &ylen, NULL);
     const int ybot = rylen - 1 + !!(r->ropts.bordermask & NCBOXMASK_BOTTOM);
     // FIXME want to tighten down whenever we're at the bottom, and the reel
     // is full, not just in this case (this can leave a gap of more than 1 row)
-    if(yoff + ylen + 1 >= ybot){
+    if(yoff + (int)ylen + 1 >= ybot){
       if(tighten_reel_down(r, ybot)){
         return -1;
       }
@@ -589,31 +667,6 @@ tighten_reel(ncreel* r){
   return trim_reel_overhang(r, top, bottom);
 }
 
-// destroy all existing tablet planes pursuant to redraw
-static void
-clean_reel(ncreel* r){
-  nctablet* vft = r->vft;
-  if(vft){
-    for(nctablet* n = vft->next ; n->p && n != vft ; n = n->next){
-//fprintf(stderr, "CLEANING NEXT: %p (%p)\n", n, n->p);
-      ncplane_destroy_family(n->p);
-      n->p = NULL;
-      n->cbp = NULL;
-    }
-    for(nctablet* n = vft->prev ; n->p && n != vft ; n = n->prev){
-//fprintf(stderr, "CLEANING PREV: %p (%p)\n", n, n->p);
-      ncplane_destroy_family(n->p);
-      n->p = NULL;
-      n->cbp = NULL;
-    }
-//fprintf(stderr, "CLEANING VFT: %p (%p)\n", vft, vft->p);
-    ncplane_destroy_family(vft->p);
-    vft->p = NULL;
-    vft->cbp = NULL;
-    r->vft = NULL;
-  }
-}
-
 // Arrange the panels, starting with the focused window, wherever it may be.
 // If necessary, resize it to the full size of the reel--focus has its
 // privileges. We then work in the opposite direction of travel, filling out
@@ -669,7 +722,7 @@ int ncreel_redraw(ncreel* nr){
   if(focused){
 //fprintf(stderr, "drawing focused tablet %p dir: %d fulcrum: %d!\n", focused, nr->direction, fulcrum);
     if(ncreel_draw_tablet(nr, focused, fulcrum, fulcrum, DIRECTION_DOWN)){
-      logerror("Error drawing tablet\n");
+      logerror("error drawing tablet");
       return -1;
     }
 //fprintf(stderr, "drew focused tablet %p -> %p lastdir: %d!\n", focused, focused->p, nr->direction);
@@ -681,25 +734,25 @@ int ncreel_redraw(ncreel* nr){
     if(nr->direction == LASTDIRECTION_DOWN){
       otherend = draw_previous_tablets(nr, otherend, &frontiertop, frontierbottom);
       if(otherend == NULL){
-        logerror("Error drawing higher tablets\n");
+        logerror("error drawing higher tablets");
         return -1;
       }
       otherend = draw_following_tablets(nr, otherend, frontiertop, &frontierbottom);
     }else{ // DIRECTION_UP
       otherend = draw_previous_tablets(nr, otherend, &frontiertop, frontierbottom);
       if(otherend == NULL){
-        logerror("Error drawing higher tablets\n");
+        logerror("error drawing higher tablets");
         return -1;
       }
       otherend = draw_following_tablets(nr, otherend, frontiertop, &frontierbottom);
     }
     if(otherend == NULL){
-      logerror("Error drawing following tablets\n");
+      logerror("error drawing following tablets");
       return -1;
     }
 //notcurses_debug(ncplane_notcurses(nr->p), stderr);
     if(tighten_reel(nr)){
-      logerror("Error tightening reel\n");
+      logerror("error tightening reel");
       return -1;
     }
 //notcurses_debug(ncplane_notcurses(nr->p), stderr);
@@ -707,7 +760,7 @@ int ncreel_redraw(ncreel* nr){
   nr->vft = nr->tablets; // update the visually-focused tablet pointer
 //fprintf(stderr, "DONE ARRANGING\n");
   if(draw_ncreel_borders(nr)){
-    logerror("Error drawing reel borders\n");
+    logerror("error drawing reel borders");
     return -1; // enforces specified dimensional minima
   }
   return 0;
@@ -719,11 +772,11 @@ validate_ncreel_opts(ncplane* n, const ncreel_options* ropts){
     return false;
   }
   if(ropts->flags >= (NCREEL_OPTION_CIRCULAR << 1u)){
-    logwarn("Provided unsupported flags 0x%016jx\n", (uintmax_t)ropts->flags);
+    logwarn("provided unsupported flags 0x%016" PRIx64, ropts->flags);
   }
   if(ropts->flags & NCREEL_OPTION_CIRCULAR){
     if(!(ropts->flags & NCREEL_OPTION_INFINITESCROLL)){
-      logerror("Can't set circular without infinitescroll\n");
+      logerror("can't set circular without infinitescroll");
       return false; // can't set circular without infinitescroll
     }
   }
@@ -733,11 +786,11 @@ validate_ncreel_opts(ncplane* n, const ncreel_options* ropts){
                             NCBOXMASK_TOP |
                             NCBOXMASK_BOTTOM;
   if(ropts->bordermask > fullmask){
-    logerror("Bad bordermask: 0x%016x\n", ropts->bordermask);
+    logerror("bad bordermask: 0x%016x", ropts->bordermask);
     return false;
   }
   if(ropts->tabletmask > fullmask){
-    logerror("Bad tabletmask: 0x%016x\n", ropts->bordermask);
+    logerror("bad tabletmask: 0x%016x", ropts->bordermask);
     return false;
   }
   return true;
@@ -747,16 +800,12 @@ ncplane* nctablet_plane(nctablet* t){
   return t->cbp;
 }
 
-ncplane* nctablet_ncplane(nctablet* t){ // deprecated
-  return nctablet_plane(t);
-}
-
 ncplane* ncreel_plane(ncreel* nr){
   return nr->p;
 }
 
 ncreel* ncreel_create(ncplane* n, const ncreel_options* ropts){
-  ncreel_options zeroed = {};
+  ncreel_options zeroed = {0};
   ncreel* nr;
   if(!ropts){
     ropts = &zeroed;
@@ -773,6 +822,11 @@ ncreel* ncreel_create(ncplane* n, const ncreel_options* ropts){
   memcpy(&nr->ropts, ropts, sizeof(*ropts));
   nr->p = n;
   nr->vft = NULL;
+  if(ncplane_set_widget(nr->p, nr, (void(*)(void*))ncreel_destroy)){
+    ncplane_destroy(nr->p);
+    free(nr);
+    return NULL;
+  }
   if(ncreel_redraw(nr)){
     ncplane_destroy(nr->p);
     free(nr);
@@ -786,7 +840,7 @@ nctablet* ncreel_add(ncreel* nr, nctablet* after, nctablet *before,
   nctablet* t;
   if(after && before){
     if(after->next != before || before->prev != after){
-      logerror("bad before (%p) / after (%p) spec\n", before, after);
+      logerror("bad before (%p) / after (%p) spec", before, after);
       return NULL;
     }
   }else if(!after && !before){
@@ -823,38 +877,15 @@ nctablet* ncreel_add(ncreel* nr, nctablet* after, nctablet *before,
   return t;
 }
 
-int ncreel_del(ncreel* nr, struct nctablet* t){
-  if(t == NULL){
-    return -1;
-  }
-  t->prev->next = t->next;
-  if(nr->tablets == t){
-    if((nr->tablets = t->next) == t){
-      nr->tablets = NULL;
-    }
-    // FIXME ought set direction based on actual location of replacement tablet
-    nr->direction = LASTDIRECTION_DOWN;
-  }
-  if(nr->vft == t){
-    clean_reel(nr); // maybe just make nr->tablets the vft?
-  }
-  t->next->prev = t->prev;
-  if(t->p){
-    ncplane_destroy_family(t->p);
-  }
-  free(t);
-  --nr->tabletcount;
-  ncreel_redraw(nr);
-  return 0;
-}
-
 void ncreel_destroy(ncreel* nreel){
   if(nreel){
-    nctablet* t;
-    while( (t = nreel->tablets) ){
-      ncreel_del(nreel, t);
+    if(ncplane_set_widget(nreel->p, NULL, NULL) == 0){
+      nctablet* t;
+      while( (t = nreel->tablets) ){
+        ncreel_del(nreel, t);
+      }
+      ncplane_destroy(nreel->p);
     }
-    ncplane_destroy(nreel->p);
     free(nreel);
   }
 }
@@ -892,6 +923,9 @@ nctablet* ncreel_prev(ncreel* nr){
 }
 
 bool ncreel_offer_input(ncreel* n, const ncinput* nc){
+  if(nc->evtype == NCTYPE_RELEASE){
+    return false;
+  }
   if(nc->id == NCKEY_UP){
     ncreel_prev(n);
     return true;
