@@ -477,16 +477,6 @@ sixelband_extend(char* vec, struct band_extender* bes, int dimx, int curx){
   return vec;
 }
 
-// get the index into the auxvec (2 bytes per pixel) given the true y/x pixel
-// coordinates, plus the origin+dimensions of the relevant cell.
-static inline int
-auxvec_idx(int y, int x, int cellpxy, int cellpxx){
-  const int xoff = x % cellpxx;
-  const int yoff = y % cellpxy;
-  const int off = yoff * cellpxx + xoff;
-  return AUXVECELEMSIZE * off;
-}
-
 // write to this cell's auxvec, backing up the pixels cleared by a wipe. wipes
 // are requested at cell granularity, broken down into sixelbands, broken down
 // by color, and then finally effected at the sixel RLE level. we're thus in
@@ -997,7 +987,7 @@ build_sixel_band(qstate* qs, int bnum){
       }
       int cidx = find_color(qs, *rgb);
       if(cidx < 0){
-        // FIXME free?
+        free(meta);
         return -1;
       }
       int act;
@@ -1022,6 +1012,7 @@ build_sixel_band(qstate* qs, int bnum){
       }else{
         b->vecs[c] = sixelband_extend(b->vecs[c], &meta[c], qs->lenx, x);
         if(b->vecs[c] == NULL){
+          free(meta);
           return -1;
         }
         meta[c].rle = 1;
@@ -1034,6 +1025,7 @@ build_sixel_band(qstate* qs, int bnum){
     if(meta[i].rle){ // color was wholly unused iff rle == 0 at end
       b->vecs[i] = sixelband_extend(b->vecs[i], &meta[i], qs->lenx, x);
       if(b->vecs[i] == NULL){
+        free(meta);
         return -1;
       }
     }else{
