@@ -8,6 +8,40 @@
 #include "windows.h"
 #include "linux.h"
 
+// tlen -- size of escape table. tused -- used bytes in same.
+// returns -1 if the starting location is >= 65535. otherwise,
+// copies tstr into the table, and sets up 1-biased index.
+int grow_esc_table(tinfo* ti, const char* tstr, escape_e esc,
+                   size_t* tlen, size_t* tused){
+  // the actual table can grow past 64KB, but we can't start there, as
+  // we only have 16-bit indices.
+  if(*tused >= 65535){
+    fprintf(stderr, "Can't add escape %d to full table\n", esc);
+    return -1;
+  }
+  if(get_escape(ti, esc)){
+    fprintf(stderr, "Already defined escape %d (%s)\n",
+            esc, get_escape(ti, esc));
+    return -1;
+  }
+  size_t slen = strlen(tstr) + 1; // count the nul term
+  if(*tlen - *tused < slen){
+    // guaranteed to give us enough space to add tstr (and then some)
+    size_t newsize = *tlen + 4020 + slen; // don't pull two pages ideally
+    char* tmp = realloc(ti->esctable, newsize);
+    if(tmp == NULL){
+      return -1;
+    }
+    ti->esctable = tmp;
+    *tlen = newsize;
+  }
+  // we now are guaranteed sufficient space to copy tstr
+  memcpy(ti->esctable + *tused, tstr, slen);
+  ti->escindices[esc] = *tused + 1; // one-bias
+  *tused += slen;
+  return 0;
+}
+
 // there does not exist any true standard terminal size. with that said, we
 // need assume *something* for the case where we're not actually attached to
 // a terminal (mainly unit tests, but also daemon environments). in preference
@@ -141,11 +175,11 @@ setup_fbcon_bitmaps(tinfo* ti, int fd){
 
 static bool
 query_rgb(void){
-  bool rgb = (tigetflag("RGB") > 1 || tigetflag("Tc") > 1);
+  bool rgb = (tigetflag("RGB") > 0 || tigetflag("Tc") > 0);
   if(!rgb){
     // RGB terminfo capability being a new thing (as of ncurses 6.1), it's not
     // commonly found in terminal entries today. COLORTERM, however, is a
-    // de-facto (if imperfect/kludgy) standard way of indicating TrueColor
+    // de facto (if imperfect/kludgy) standard way of indicating TrueColor
     // support for a terminal. The variable takes one of two case-sensitive
     // values:
     //
@@ -351,9 +385,9 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 #define KITTYQUERY
 #endif
 
-// request kitty keyboard protocol features 1, 2, and 8, first pushing current.
+// request kitty keyboard protocol features 1, 2, 8 and 16, first pushing current.
 // see https://sw.kovidgoyal.net/kitty/keyboard-protocol/#progressive-enhancement
-#define KKBDSUPPORT "\x1b[=11u"
+#define KKBDSUPPORT "\x1b[=27u"
 
 // the kitty keyboard protocol allows unambiguous, complete identification of
 // input events. this queries for the level of support. we want to do this
@@ -707,6 +741,7 @@ apply_kitty_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused){
   if(add_smulx_escapes(ti, tablelen, tableused)){
     return NULL;
   }
+  // FIXME we have problems using SELFREF; fix them!
   /*if(compare_versions(ti->termversion, "0.22.1") >= 0){
     setup_kitty_bitmaps(ti, ti->ttyfd, NCPIXEL_KITTY_SELFREF);
   }else*/ if(compare_versions(ti->termversion, "0.20.0") >= 0){
@@ -758,7 +793,8 @@ apply_vte_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused){
 }
 
 static const char*
-apply_foot_heuristics(tinfo* ti, bool* forcesdm, bool* invertsixel){
+apply_foot_heuristics(tinfo* ti, size_t *tablelen, size_t *tableused,
+                      bool* forcesdm, bool* invertsixel){
   ti->caps.sextants = true;
   ti->caps.quadrants = true;
   ti->caps.rgb = true;
@@ -766,6 +802,14 @@ apply_foot_heuristics(tinfo* ti, bool* forcesdm, bool* invertsixel){
   if(compare_versions(ti->termversion, "1.8.2") < 0){
     *invertsixel = true;
   }
+  if(compare_versions(ti->termversion, "1.18.0") >= 0){
+    if(add_smulx_escapes(ti, tablelen, tableused)){
+      return NULL;
+    }
+  }
+  if(compare_versions(ti->termversion, "1.20.0") >= 0){
+    ti->caps.octants = true;
+  }
   return "foot";
 }
 
@@ -834,12 +878,19 @@ apply_msterminal_heuristics(tinfo* ti){
 }
 
 static const char*
-apply_contour_heuristics(tinfo* ti, bool* forcesdm, bool* invertsixel){
+apply_contour_heuristics(tinfo* ti, size_t* tablelen, size_t* tableused,
+                         bool* forcesdm, bool* invertsixel){
+  if(add_smulx_escapes(ti, tablelen, tableused)){
+    return NULL;
+  }
+  if(add_pushcolors_escapes(ti, tablelen, tableused)){
+    return NULL;
+  }
   ti->caps.quadrants = true;
   ti->caps.sextants = true;
   ti->caps.rgb = true;
   *forcesdm = true;
-  *invertsixel = true;
+  *invertsixel = false;
   return "Contour";
 }
 
@@ -875,6 +926,13 @@ apply_konsole_heuristics(tinfo* ti){
   return "Konsole";
 }
 
+static const char*
+apply_ghostty_heuristics(tinfo* ti){
+  ti->caps.quadrants = true;
+  ti->caps.sextants = true;
+  return "ghostty";
+}
+
 static const char*
 apply_linux_heuristics(tinfo* ti, unsigned nonewfonts){
   const char* tname = NULL;
@@ -925,6 +983,7 @@ apply_term_heuristics(tinfo* ti, const char* tname, queried_terminals_e qterm,
     // setupterm interprets a missing/empty TERM variable as the special value “unknown”.
     tname = ti->termname ? ti->termname : "unknown";
   }
+  loginfo("tname is %s (qterm %d)", tname, qterm);
   // st had neither caps.sextants nor caps.quadrants last i checked (0.8.4)
   ti->caps.braille = true; // most everyone has working caps.braille, even from fonts
   ti->caps.halfblocks = true; // most everyone has working halfblocks
@@ -941,7 +1000,8 @@ apply_term_heuristics(tinfo* ti, const char* tname, queried_terminals_e qterm,
       newname = apply_vte_heuristics(ti, tablelen, tableused);
       break;
     case TERMINAL_FOOT:
-      newname = apply_foot_heuristics(ti, forcesdm, invertsixel);
+      newname = apply_foot_heuristics(ti, tablelen, tableused,
+                                      forcesdm, invertsixel);
       break;
     case TERMINAL_TMUX:
       newname = "tmux"; // FIXME what, oh what to do with tmux?
@@ -967,7 +1027,8 @@ apply_term_heuristics(tinfo* ti, const char* tname, queried_terminals_e qterm,
       newname = apply_msterminal_heuristics(ti);
       break;
     case TERMINAL_CONTOUR:
-      newname = apply_contour_heuristics(ti, forcesdm, invertsixel);
+      newname = apply_contour_heuristics(ti, tablelen, tableused,
+                                         forcesdm, invertsixel);
       break;
     case TERMINAL_ITERM:
       newname = apply_iterm_heuristics(ti, tablelen, tableused);
@@ -987,7 +1048,11 @@ apply_term_heuristics(tinfo* ti, const char* tname, queried_terminals_e qterm,
     case TERMINAL_KONSOLE:
       newname = apply_konsole_heuristics(ti);
       break;
+    case TERMINAL_GHOSTTY:
+      newname = apply_ghostty_heuristics(ti);
+      break;
     default:
+      logwarn("no match for qterm %d tname %s", qterm, tname);
       newname = tname;
       break;
   }
@@ -1004,6 +1069,10 @@ apply_term_heuristics(tinfo* ti, const char* tname, queried_terminals_e qterm,
   if(wcwidth(L'🬸') < 0){
     ti->caps.sextants = false;
   }
+  // run a wcwidth(𜴀) to guarantee libc Unicode 16 support, independent of term
+  if(wcwidth(L'𜴀') < 0){
+    ti->caps.octants = false;
+  }
   ti->termname = tname;
   return 0;
 }
@@ -1295,6 +1364,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
 #endif
 #endif
   if(ti->ttyfd >= 0){
+#ifndef __MINGW32__
     if((ti->tpreserved = calloc(1, sizeof(*ti->tpreserved))) == NULL){
       return -1;
     }
@@ -1309,6 +1379,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
       free(ti->tpreserved);
       return -1;
     }
+#endif
     // if we already know our terminal (e.g. on the linux console), there's no
     // need to send the identification queries. the controls are sufficient.
     bool minimal = (ti->qterm != TERMINAL_UNKNOWN);
@@ -1326,13 +1397,15 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     goto err;
   }
   tname = termname(); // longname() is also available
-#endif
   int linesigs_enabled = 1;
   if(ti->tpreserved){
     if(!(ti->tpreserved->c_lflag & ISIG)){
       linesigs_enabled = 0;
     }
   }
+#else
+  int linesigs_enabled = 0;
+#endif
   if(init_inputlayer(ti, stdin, lmargin, tmargin, rmargin, bmargin,
                      stats, draininput, linesigs_enabled)){
     goto err;
@@ -1583,25 +1656,27 @@ int cbreak_mode(tinfo* ti){
 }
 
 // replace or populate the TERM environment variable with 'termname'
+// (for the benefit of subprocesses)
+#define ENVVAR "TERM"
 int putenv_term(const char* tname){
-  #define ENVVAR "TERM"
   const char* oldterm = getenv(ENVVAR);
   if(oldterm){
-    logdebug("replacing %s value %s with %s", ENVVAR, oldterm, tname);
+    if(strcmp(oldterm, tname) == 0){
+      return 0;
+    }
+    logdebug("replacing " ENVVAR " value %s with %s", oldterm, tname);
   }else{
-    loginfo("provided %s value %s", ENVVAR, tname);
+    loginfo("providing " ENVVAR "=%s", tname);
   }
-  if(oldterm && strcmp(oldterm, tname) == 0){
-    return 0;
-  }
-  char* buf = malloc(strlen(tname) + strlen(ENVVAR) + 1);
-  if(buf == NULL){
+#ifndef __MINGW32__
+  if(setenv(ENVVAR, tname, 1)){
+#else
+  SetEnvironmentVariable(ENVVAR, NULL); // clear it from environment
+  if(!SetEnvironmentVariable(ENVVAR, tname)){ // set it
+#endif
+    logerror("error exporting " ENVVAR "=%s (%s)", tname, strerror(errno));
     return -1;
   }
-  int c = putenv(buf);
-  if(c){
-    logerror("couldn't export %s", buf);
-  }
-  free(buf);
-  return c;
+  return 0;
 }
+#undef ENVVAR
