@@ -122,7 +122,9 @@ ncvisual* ncvisual_create(void){
     return visual_implementation->visual_create();
   }
   ncvisual* ret = malloc(sizeof(*ret));
-  memset(ret, 0, sizeof(*ret));
+  if(ret){
+    memset(ret, 0, sizeof(*ret));
+  }
   return ret;
 }
 
@@ -469,6 +471,7 @@ void* rgb_loose_to_rgba(const void* data, int rows, int* rowstride, int cols, in
       for(int x = 0 ; x < cols ; ++x){
         const uint32_t* src = (const uint32_t*)data + (*rowstride / 4) * y + x;
         uint32_t* dst = ret + cols * y + x;
+        *dst = 0; // kill scan-build warning about using uninitialized value below
         ncpixel_set_a(dst, alpha);
         ncpixel_set_r(dst, ncpixel_r(*src));
         ncpixel_set_g(dst, ncpixel_g(*src));
@@ -490,6 +493,7 @@ void* rgb_packed_to_rgba(const void* data, int rows, int* rowstride, int cols, i
       for(int x = 0 ; x < cols ; ++x){
         const unsigned char* src = (const unsigned char*)data + *rowstride * y + x;
         uint32_t* dst = ret + cols * y + x;
+        *dst = 0; // kill scan-build warning about using uninitialized value below
         ncpixel_set_a(dst, alpha);
         ncpixel_set_r(dst, src[0]);
         ncpixel_set_g(dst, src[1]);
@@ -514,6 +518,7 @@ void* bgra_to_rgba(const void* data, int rows, int* rowstride, int cols, int alp
       for(int x = 0 ; x < cols ; ++x){
         const uint32_t* src = (const uint32_t*)data + (*rowstride / 4) * y + x;
         uint32_t* dst = ret + cols * y + x;
+        *dst = 0; // kill scan-build warning about using uninitialized value below
         ncpixel_set_a(dst, alpha);
         ncpixel_set_r(dst, ncpixel_b(*src));
         ncpixel_set_g(dst, ncpixel_g(*src));
@@ -776,6 +781,10 @@ ncvisual* ncvisual_from_rgba(const void* rgba, int rows, int rowstride, int cols
     logerror("rowstride %d not a multiple of 4", rowstride);
     return NULL;
   }
+  if(rowstride * 4 < cols || cols <= 0 || rows <= 0){
+    logerror("invalid rowstride or geometry");
+    return NULL;
+  }
   ncvisual* ncv = ncvisual_create();
   if(ncv){
     // ffmpeg needs inputs with rows aligned on 192-byte boundaries
@@ -810,6 +819,14 @@ ncvisual* ncvisual_from_sixel(const char* s, unsigned leny, unsigned lenx){
 
 ncvisual* ncvisual_from_rgb_packed(const void* rgba, int rows, int rowstride,
                                    int cols, int alpha){
+  if(rowstride % 3){
+    logerror("rowstride %d not a multiple of 3", rowstride);
+    return NULL;
+  }
+  if(rows <= 0 || cols <= 0 || rowstride < cols * 3){
+    logerror("illegal packed rgb geometry");
+    return NULL;
+  }
   ncvisual* ncv = ncvisual_create();
   if(ncv){
     ncv->rowstride = pad_for_image(cols * 4, cols);
@@ -828,6 +845,7 @@ ncvisual* ncvisual_from_rgb_packed(const void* rgba, int rows, int rowstride,
         memcpy(&r, src + rowstride * y + 3 * x, 1);
         memcpy(&g, src + rowstride * y + 3 * x + 1, 1);
         memcpy(&b, src + rowstride * y + 3 * x + 2, 1);
+        data[y * ncv->rowstride / 4 + x] = 0; // eliminate scan-build uninitialized data warning
         ncpixel_set_a(&data[y * ncv->rowstride / 4 + x], alpha);
         ncpixel_set_r(&data[y * ncv->rowstride / 4 + x], r);
         ncpixel_set_g(&data[y * ncv->rowstride / 4 + x], g);
@@ -847,6 +865,10 @@ ncvisual* ncvisual_from_rgb_loose(const void* rgba, int rows, int rowstride,
     logerror("rowstride %d not a multiple of 4", rowstride);
     return NULL;
   }
+  if(rows <= 0 || cols <= 0 || rowstride < cols * 4){
+    logerror("illegal packed rgb geometry");
+    return NULL;
+  }
   ncvisual* ncv = ncvisual_create();
   if(ncv){
     ncv->rowstride = pad_for_image(cols * 4, cols);
@@ -872,6 +894,11 @@ ncvisual* ncvisual_from_rgb_loose(const void* rgba, int rows, int rowstride,
 
 ncvisual* ncvisual_from_bgra(const void* bgra, int rows, int rowstride, int cols){
   if(rowstride % 4){
+    logerror("rowstride %d not a multiple of 4", rowstride);
+    return NULL;
+  }
+  if(rows <= 0 || cols <= 0 || rowstride < cols * 4){
+    logerror("illegal bgra geometry");
     return NULL;
   }
   ncvisual* ncv = ncvisual_create();
@@ -889,6 +916,7 @@ ncvisual* ncvisual_from_bgra(const void* bgra, int rows, int rowstride, int cols
         uint32_t src;
         memcpy(&src, (const char*)bgra + y * rowstride + x * 4, 4);
         uint32_t* dst = &data[ncv->rowstride * y / 4 + x];
+        *dst = 0; // kill scan-build warning about using uninitialized value below
         ncpixel_set_a(dst, ncpixel_a(src));
         ncpixel_set_r(dst, ncpixel_b(src));
         ncpixel_set_g(dst, ncpixel_g(src));
@@ -905,10 +933,14 @@ ncvisual* ncvisual_from_bgra(const void* bgra, int rows, int rowstride, int cols
 ncvisual* ncvisual_from_palidx(const void* pdata, int rows, int rowstride,
                                int cols, int palsize, int pstride,
                                const uint32_t* palette){
-  if(rowstride % pstride){
+  if(pstride <= 0 || rowstride % pstride){
     logerror("bad pstride (%d) for rowstride (%d)", pstride, rowstride);
     return NULL;
   }
+  if(rows <= 0 || cols <= 0 || rowstride < cols * pstride){
+    logerror("illegal palimg geometry");
+    return NULL;
+  }
   if(palsize > 256 || palsize <= 0){
     logerror("palettes size (%d) is unsupported", palsize);
     return NULL;
@@ -935,6 +967,7 @@ ncvisual* ncvisual_from_palidx(const void* pdata, int rows, int rowstride,
         uint32_t src = palette[palidx];
         uint32_t* dst = &data[ncv->rowstride * y / 4 + x];
         if(ncchannel_default_p(src)){
+          *dst = 0; // kill scan-build warning about using uninitialized value below
           // FIXME use default color as detected, or just 0xffffff
           ncpixel_set_a(dst, 255 - palidx);
           ncpixel_set_r(dst, palidx);
