@@ -16,7 +16,7 @@ static void
 usage(const char* argv0, FILE* o){
   fprintf(o, "usage: %s [ -hVq ] files\n", argv0);
   fprintf(o, " -h: print help and return success\n");
-  fprintf(o, " -v: print version and return success\n");
+  fprintf(o, " -V: print version and return success\n");
   fprintf(o, " -q: don't wait for any input\n");
 }
 
@@ -286,7 +286,6 @@ putpara(struct ncplane* p, const char* text){
           curend = macend;
           break;
         }else{
-          inescape = false;
           cur = curend++;
           break;
         }
@@ -613,6 +612,9 @@ manloop(struct notcurses* nc, const char* arg, unsigned noui){
         newy = ncplane_y(page) - (int)ncplane_dim_y(stdn) + 1;
         movedown = true;
         break;
+      case 'g': case NCKEY_HOME:
+        newy = 1;
+        break;
       }case 'q':
         ret = 0;
         goto done;
@@ -667,7 +669,7 @@ int main(int argc, char** argv){
   if(nc == NULL){
     return EXIT_FAILURE;
   }
-  bool success;
+  bool success = false;
   for(int i = 0 ; i < argc - nonopt ; ++i){
     success = false;
     if(tfman(nc, argv[nonopt + i], noui)){
