@@ -9,6 +9,7 @@
 #include <libavutil/rational.h>
 #include <libswscale/swscale.h>
 #include <libswscale/version.h>
+#include <libavdevice/avdevice.h>
 #include <libavformat/version.h>
 #include <libavformat/avformat.h>
 #include "lib/visual-details.h"
@@ -39,6 +40,14 @@ typedef struct ncvisual_details {
 
 #define IMGALLOCALIGN 64
 
+uint64_t ffmpeg_pkt_duration(const AVFrame* frame){
+#if LIBAVUTIL_VERSION_MAJOR < 58
+      return frame->pkt_duration;
+#else
+      return frame->duration;
+#endif
+}
+
 /*static void
 print_frame_summary(const AVCodecContext* cctx, const AVFrame* f){
   if(f == NULL){
@@ -75,7 +84,7 @@ print_frame_summary(const AVCodecContext* cctx, const AVFrame* f){
   }
   fprintf(stderr, " PTS %" PRId64 " Flags: 0x%04x\n", f->pts, f->flags);
   fprintf(stderr, " %" PRIu64 "ms@%" PRIu64 "ms (%skeyframe) qual: %d\n",
-          f->pkt_duration, // FIXME in 'time_base' units
+          ffmpeg_pkt_duration(f), // FIXME in 'time_base' units
           f->best_effort_timestamp,
           f->key_frame ? "" : "non-",
           f->quality);
@@ -480,12 +489,11 @@ static int
 ffmpeg_stream(notcurses* nc, ncvisual* ncv, float timescale,
               ncstreamcb streamer, const struct ncvisual_options* vopts,
               void* curry){
-  int frame = 1;
   struct timespec begin; // time we started
   clock_gettime(CLOCK_MONOTONIC, &begin);
   uint64_t nsbegin = timespec_to_ns(&begin);
   //bool usets = false;
-  // each frame has a pkt_duration in milliseconds. keep the aggregate, in case
+  // each frame has a duration in units of time_base. keep the aggregate, in case
   // we don't have PTS available.
   uint64_t sum_duration = 0;
   ncplane* newn = NULL;
@@ -519,8 +527,9 @@ ffmpeg_stream(notcurses* nc, ncvisual* ncv, float timescale,
     if(activevopts.n != newn){
       activevopts.n = newn;
     }
-    ++frame;
-    uint64_t duration = ncv->details->frame->pkt_duration * tbase * NANOSECS_IN_SEC;
+    // display duration in units of time_base
+    const uint64_t pktduration = ffmpeg_pkt_duration(ncv->details->frame);
+    uint64_t duration = pktduration * tbase * NANOSECS_IN_SEC;
     double schedns = nsbegin;
     sum_duration += (duration * timescale);
     schedns += sum_duration;
@@ -691,22 +700,24 @@ ffmpeg_log_level(int level){
 static int
 ffmpeg_init(int logl){
   av_log_set_level(ffmpeg_log_level(logl));
+  avdevice_register_all();
   // FIXME could also use av_log_set_callback() and capture the message...
   return 0;
 }
 
 static void
 ffmpeg_printbanner(fbuf* f){
-  fbuf_printf(f, "avformat %u.%u.%u avutil %u.%u.%u swscale %u.%u.%u avcodec %u.%u.%u" NL,
+  fbuf_printf(f, "avformat %u.%u.%u avutil %u.%u.%u swscale %u.%u.%u avcodec %u.%u.%u avdevice %u.%u.%u" NL,
               LIBAVFORMAT_VERSION_MAJOR, LIBAVFORMAT_VERSION_MINOR, LIBAVFORMAT_VERSION_MICRO,
               LIBAVUTIL_VERSION_MAJOR, LIBAVUTIL_VERSION_MINOR, LIBAVUTIL_VERSION_MICRO,
               LIBSWSCALE_VERSION_MAJOR, LIBSWSCALE_VERSION_MINOR, LIBSWSCALE_VERSION_MICRO,
-              LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO);
+              LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO,
+              LIBAVDEVICE_VERSION_MAJOR, LIBAVDEVICE_VERSION_MINOR, LIBAVDEVICE_VERSION_MICRO);
 }
 
 static void
 ffmpeg_details_destroy(ncvisual_details* deets){
-  avcodec_close(deets->codecctx);
+  // avcodec_close() is deprecated; avcodec_free_context() suffices
   avcodec_free_context(&deets->subtcodecctx);
   avcodec_free_context(&deets->codecctx);
   av_frame_free(&deets->frame);
