@@ -4,6 +4,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <pthread.h>
+#include <compat/compat.h>
 #include <notcurses/notcurses.h>
 
 static bool fddone;
@@ -42,8 +43,9 @@ int main(int argc, char** argv){
   }
   setlocale(LC_ALL, "");
   notcurses_options opts = {
-    .flags = NCOPTION_INHIBIT_SETLOCALE |
-              NCOPTION_SUPPRESS_BANNERS,
+    .flags = NCOPTION_INHIBIT_SETLOCALE
+             | NCOPTION_SUPPRESS_BANNERS
+             | NCOPTION_DRAIN_INPUT,
   };
   struct notcurses* nc = notcurses_core_init(&opts, NULL);
   struct ncplane* n = notcurses_stdplane(nc);
@@ -54,7 +56,7 @@ int main(int argc, char** argv){
       fprintf(stderr, "Couldn't open %s (%s)\n", *argv, strerror(errno));
       goto done;
     }
-    ncfdplane_options nopts = {};
+    ncfdplane_options nopts = {0};
     struct ncfdplane* ncfp = ncfdplane_create(n, &nopts, fd, cb, eofcb);
     pthread_mutex_lock(&lock);
     while(!fddone){
