@@ -28,14 +28,22 @@ run_menu(struct notcurses* nc, struct ncmenu* ncm){
   uint32_t keypress;
   ncinput ni;
   notcurses_render(nc);
-  while((keypress = notcurses_getc_blocking(nc, &ni)) != (uint32_t)-1){
+  while((keypress = notcurses_get_blocking(nc, &ni)) != (uint32_t)-1){
     if(!ncmenu_offer_input(ncm, &ni)){
-      if(keypress == 'q'){
+      const char* sel;
+      if( (sel = ncmenu_mouse_selected(ncm, &ni, NULL)) ){
+        if(sel && !strcmp(sel, "Quit")){
+          ncmenu_destroy(ncm);
+          ncplane_destroy(selplane);
+          return 0;
+        }
+      }else if(ni.evtype == NCTYPE_RELEASE){
+        continue;
+      }else if(keypress == 'q'){
         ncmenu_destroy(ncm);
         ncplane_destroy(selplane);
         return 0;
       }else if(keypress == NCKEY_ENTER){ // selected a menu item
-        const char* sel;
         if( (sel = ncmenu_selected(ncm, &ni)) ){
           if(strcmp(sel, "Quit") == 0){
             ncmenu_destroy(ncm);
@@ -60,42 +68,37 @@ err:
 }
 
 int main(void){
-  if(!setlocale(LC_ALL, "")){
-    return EXIT_FAILURE;
-  }
-  notcurses_options opts = {
-    .flags = NCOPTION_INHIBIT_SETLOCALE,
-  };
+  notcurses_options opts = { };
   struct notcurses* nc = notcurses_core_init(&opts, NULL);
   if(nc == NULL){
     return EXIT_FAILURE;
   }
-  notcurses_mouse_enable(nc);
+  notcurses_mice_enable(nc, NCMICE_BUTTON_EVENT);
   struct ncmenu_item demo_items[] = {
-    { .desc = "Restart", .shortcut = { .id = 'r', .ctrl = true, }, },
-    { .desc = "Disabled", .shortcut = { .id = 'd', .ctrl = false, }, },
+    { .desc = "Restart", .shortcut = { .id = 'r', .modifiers = NCKEY_MOD_CTRL, }, },
+    { .desc = "Derp", .shortcut = { .id = 'd', }, },
   };
   struct ncmenu_item file_items[] = {
-    { .desc = "New", .shortcut = { .id = 'n', .ctrl = true, }, },
-    { .desc = "Open", .shortcut = { .id = 'o', .ctrl = true, }, },
-    { .desc = "Close", .shortcut = { .id = 'c', .ctrl = true, }, },
+    { .desc = "New", .shortcut = { .id = 'n', .modifiers = NCKEY_MOD_CTRL, }, },
+    { .desc = "Open", .shortcut = { .id = 'o', .modifiers = NCKEY_MOD_CTRL, }, },
+    { .desc = "Close", .shortcut = { .id = 'c', .modifiers = NCKEY_MOD_CTRL, }, },
     { .desc = NULL, },
     { .desc = "Quit", .shortcut = { .id = 'q', }, },
   };
   struct ncmenu_item help_items[] = {
-    { .desc = "About", .shortcut = { .id = 'a', .ctrl = true, }, },
+    { .desc = "About", .shortcut = { .id = 'a', .modifiers = NCKEY_MOD_CTRL, }, },
   };
   struct ncmenu_section sections[] = {
     { .name = "Schwarzgerät", .items = demo_items,
       .itemcount = sizeof(demo_items) / sizeof(*demo_items),
-      .shortcut = { .id = 0x00e4, .alt = true, }, },
+      .shortcut = { .id = 0x00e4, .modifiers = NCKEY_MOD_ALT, }, },
     { .name = "File", .items = file_items,
       .itemcount = sizeof(file_items) / sizeof(*file_items),
-      .shortcut = { .id = 'f', .alt = true, }, },
+      .shortcut = { .id = 'f', .modifiers = NCKEY_MOD_ALT, }, },
     { .name = NULL, .items = NULL, .itemcount = 0, },
     { .name = "Help", .items = help_items,
       .itemcount = sizeof(help_items) / sizeof(*help_items),
-      .shortcut = { .id = 'h', .alt = true, }, },
+      .shortcut = { .id = 'h', .modifiers = NCKEY_MOD_ALT, }, },
   };
   ncmenu_options mopts;
   memset(&mopts, 0, sizeof(mopts));
@@ -105,16 +108,16 @@ int main(void){
   ncchannels_set_bg_rgb(&mopts.headerchannels, 0x440000);
   ncchannels_set_fg_rgb(&mopts.sectionchannels, 0xb0d700);
   ncchannels_set_bg_rgb(&mopts.sectionchannels, 0x002000);
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stddim_yx(nc, &dimy, &dimx);
   struct ncmenu* top = ncmenu_create(n, &mopts);
   if(top == NULL){
     goto err;
   }
-  if(ncmenu_item_set_status(top, "Schwarzgerät", "Disabled", false)){
+  if(ncmenu_item_set_status(top, "Schwarzgerät", "Restart", false)){
     goto err;
   }
-  if(ncmenu_item_set_status(top, "Schwarzgerät", "Restart", false)){
+  if(ncmenu_item_set_status(top, "File", "Open", false)){
     goto err;
   }
   uint64_t channels = 0;
@@ -138,6 +141,12 @@ int main(void){
   if(bottom == NULL){
     goto err;
   }
+  if(ncmenu_item_set_status(bottom, "Schwarzgerät", "Restart", false)){
+    goto err;
+  }
+  if(ncmenu_item_set_status(bottom, "Schwarzgerät", "Derp", false)){
+    goto err;
+  }
   if(ncplane_putstr_aligned(n, 0, NCALIGN_RIGHT, " -=+ menu poc. press q to exit +=- ") < 0){
 	  return EXIT_FAILURE;
   }
