@@ -6,7 +6,7 @@
 #include "version.h"
 
 static struct ncselector_item items[] = {
-#define SITEM(short, long) { short, long, 0, 0, }
+#define SITEM(s, l) { s, l, }
   SITEM("Afrikaans", "Ek kan glas eet, dit maak my nie seer nie."),
   SITEM("AngloSax", "ᛁᚳ᛫ᛗᚨᚷ᛫ᚷᛚᚨᛋ᛫ᛖᚩᛏᚪᚾ᛫ᚩᚾᛞ᛫ᚻᛁᛏ᛫ᚾᛖ᛫ᚻᛖᚪᚱᛗᛁᚪᚧ᛫ᛗᛖ᛬"),
   SITEM("Japanese", "私はガラスを食べられます。それは私を傷つけません。"),
@@ -33,11 +33,18 @@ run_selector(struct notcurses* nc, struct ncselector* ns){
   notcurses_render(nc);
   uint32_t keypress;
   ncinput ni;
-  while((keypress = notcurses_getc_blocking(nc, &ni)) != (uint32_t)-1){
+  while((keypress = notcurses_get_blocking(nc, &ni)) != (uint32_t)-1){
     if(!ncselector_offer_input(ns, &ni)){
+      if(ni.evtype == NCTYPE_RELEASE){
+        continue;
+      }
       switch(keypress){
         case NCKEY_ENTER: ncselector_destroy(ns, NULL); return;
-        case 'M': case 'J': if(ni.ctrl){ ncselector_destroy(ns, NULL); return; }
+        case 'M': case 'J':
+          if(ncinput_ctrl_p(&ni)){
+            ncselector_destroy(ns, NULL);
+            return;
+          }
       }
       if(keypress == 'q'){
         break;
@@ -49,20 +56,12 @@ run_selector(struct notcurses* nc, struct ncselector* ns){
 }
 
 int main(void){
-  if(!setlocale(LC_ALL, "")){
-    return EXIT_FAILURE;
-  }
-  notcurses_options opts = {
-    .flags = NCOPTION_INHIBIT_SETLOCALE,
-  };
+  notcurses_options opts = { };
   struct notcurses* nc = notcurses_init(&opts, NULL);
   if(nc == NULL){
     return EXIT_FAILURE;
   }
-  if(notcurses_mouse_enable(nc)){
-    notcurses_stop(nc);
-    return EXIT_FAILURE;
-  }
+  notcurses_mice_enable(nc, NCMICE_BUTTON_EVENT);
   ncselector_options sopts;
   memset(&sopts, 0, sizeof(sopts));
   sopts.maxdisplay = 4;
@@ -90,7 +89,7 @@ int main(void){
       .scaling = NCSCALE_STRETCH,
       .n = n,
     };
-    if(ncvisual_render(nc, ncv, &vopts) == NULL){
+    if(ncvisual_blit(nc, ncv, &vopts) == NULL){
       goto err;
     }
     ncvisual_destroy(ncv);
