@@ -9,12 +9,14 @@ auto main() -> int {
     return EXIT_FAILURE;
   }
   notcurses_options opts{};
-  opts.flags = NCOPTION_INHIBIT_SETLOCALE | NCOPTION_NO_ALTERNATE_SCREEN;
+  opts.flags = NCOPTION_INHIBIT_SETLOCALE
+               | NCOPTION_NO_ALTERNATE_SCREEN
+               | NCOPTION_DRAIN_INPUT;
   struct notcurses* nc = notcurses_init(&opts, nullptr);
   if(nc == nullptr){
     return EXIT_FAILURE;
   }
-  int y, x, dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stdplane(nc);
   ncplane_dim_yx(n, &dimy, &dimx);
   int r , g, b;
@@ -22,14 +24,14 @@ auto main() -> int {
   g = 0x80;
   b = 0;
   ncplane_set_fg_rgb8(n, 0x40, 0x20, 0x40);
-  for(y = 0 ; y < dimy ; ++y){
-    for(x = 0 ; x < dimx ; ++x){
+  for(unsigned y = 0 ; y < dimy ; ++y){
+    if(ncplane_cursor_move_yx(n, y, 0)){
+      goto err;
+    }
+    for(unsigned x = 0 ; x < dimx ; ++x){
       if(ncplane_set_bg_rgb8(n, r, g, b)){
         goto err;
       }
-      if(ncplane_cursor_move_yx(n, y, x)){
-        goto err;
-      }
       if(ncplane_putchar(n, 'x') <= 0){
         goto err;
       }
