@@ -1,7 +1,6 @@
 #include "main.h"
 #include <cstdlib>
 #include <iostream>
-#include "internal.h"
 
 auto pulser(struct notcurses* nc, struct ncplane* ncp __attribute__ ((unused)),
             const struct timespec* ts __attribute__ ((unused)), void* curry) -> int {
@@ -39,14 +38,13 @@ TEST_CASE("Fade") {
     return;
   }
   REQUIRE(0 == ncplane_cursor_move_yx(n_, 0, 0));
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(n_, &dimy, &dimx);
-  nccell c = CELL_CHAR_INITIALIZER('*');
-  nccell_set_fg_rgb8(&c, 0xff, 0xff, 0xff);
+  nccell c = NCCELL_CHAR_INITIALIZER('*');
   unsigned rgb = 0xffffffu;
   CHECK(!ncplane_set_scrolling(n_, true));
-  for(int y = 0 ; y < dimy ; ++y){
-    for(int x = 0 ; x < dimx ; ++x){
+  for(unsigned y = 0 ; y < dimy ; ++y){
+    for(unsigned x = 0 ; x < dimx ; ++x){
       rgb -= 32;
       if(rgb < 32){
         rgb = 0xffffffu;
