@@ -35,8 +35,10 @@ TEST_CASE("Rotate") {
     CHECK(0 == notcurses_stop(nc_));
     return;
   }
-  int dimy, dimx;
-  struct ncplane* n_ = notcurses_stddim_yx(nc_, &dimy, &dimx);
+  unsigned udimy, udimx;
+  struct ncplane* n_ = notcurses_stddim_yx(nc_, &udimy, &udimx);
+  int dimy = udimy;
+  int dimx = udimx;
   REQUIRE(n_);
 
   uint64_t ul, ur, ll, lr;
@@ -67,9 +69,9 @@ TEST_CASE("Rotate") {
     CHECK(0 == ncchannels_set_fg_alpha(&channels, NCALPHA_TRANSPARENT));
     CHECK(0 == ncchannels_set_bg_alpha(&channels, NCALPHA_TRANSPARENT));
     REQUIRE(0 >= ncplane_set_base(testn, "", 0, channels));
-    nccell tl = CELL_TRIVIAL_INITIALIZER, tr = CELL_TRIVIAL_INITIALIZER;
-    nccell bl = CELL_TRIVIAL_INITIALIZER, br = CELL_TRIVIAL_INITIALIZER;
-    nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+    nccell tl = NCCELL_TRIVIAL_INITIALIZER, tr = NCCELL_TRIVIAL_INITIALIZER;
+    nccell bl = NCCELL_TRIVIAL_INITIALIZER, br = NCCELL_TRIVIAL_INITIALIZER;
+    nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(-1 < nccell_prime(testn, &tl, "█", 0, ul));
     CHECK(-1 < nccell_prime(testn, &tr, "█", 0, ur));
     CHECK(-1 < nccell_prime(testn, &bl, "█", 0, ll));
@@ -98,7 +100,7 @@ TEST_CASE("Rotate") {
       .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* testn = ncplane_create(n_, &nopts);
-    REQUIRE(0 < ncplane_gradient_sized(testn, " ", 0, ul, ur, ll, lr, 8, 16));
+    REQUIRE(0 < ncplane_gradient(testn, -1, -1, 8, 16, " ", 0, ul, ur, ll, lr));
     RotateCW(nc_, testn);
     CHECK(0 == ncplane_destroy(testn));
   }
@@ -117,7 +119,7 @@ TEST_CASE("Rotate") {
       .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* testn = ncplane_create(n_, &nopts);
-    REQUIRE(0 < ncplane_gradient_sized(testn, " ", 0, ul, ur, ll, lr, 8, 32));
+    REQUIRE(0 < ncplane_gradient(testn, -1, -1, 8, 32, " ", 0, ul, ur, ll, lr));
     RotateCW(nc_, testn);
     CHECK(0 == ncplane_destroy(testn));
   }
@@ -136,7 +138,7 @@ TEST_CASE("Rotate") {
       .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* testn = ncplane_create(n_, &nopts);
-    REQUIRE(0 < ncplane_gradient_sized(testn, " ", 0, ul, ur, ll, lr, 8, 16));
+    REQUIRE(0 < ncplane_gradient(testn, -1, -1, 8, 16, " ", 0, ul, ur, ll, lr));
     RotateCCW(nc_, testn);
     CHECK(0 == ncplane_destroy(testn));
   }
@@ -155,7 +157,7 @@ TEST_CASE("Rotate") {
       .margin_b = 0, .margin_r = 0,
     };
     struct ncplane* testn = ncplane_create(n_, &nopts);
-    REQUIRE(0 < ncplane_gradient_sized(testn, " ", 0, ul, ur, ll, lr, 8, 32));
+    REQUIRE(0 < ncplane_gradient(testn, -1, -1, 8, 32, " ", 0, ul, ur, ll, lr));
     RotateCCW(nc_, testn);
     CHECK(0 == ncplane_destroy(testn));
   }
@@ -171,11 +173,13 @@ TEST_CASE("Rotate") {
     auto ncv = ncvisual_from_rgba(rgba.data(), height, width * 4, width);
     REQUIRE(ncv);
     ncvisual_options opts{};
-    auto rendered = ncvisual_render(nc_, ncv, &opts);
+    opts.n = n_;
+    opts.flags = NCVISUAL_OPTION_CHILDPLANE;
+    auto rendered = ncvisual_blit(nc_, ncv, &opts);
     REQUIRE(rendered);
-    int pxdimy, pxdimx;
+    unsigned pxdimy, pxdimx;
     uint32_t* rgbaret = ncplane_as_rgba(rendered, NCBLIT_2x1,
-                                        0, 0, -1, -1, &pxdimy, &pxdimx);
+                                        0, 0, 0, 0, &pxdimy, &pxdimx);
     REQUIRE(rgbaret);
     if(height % 2){
       ++height;
@@ -207,16 +211,16 @@ TEST_CASE("Rotate") {
       opts.n = rendered;
       // FIXME check pixels after all rotations
       CHECK(0 == ncvisual_rotate(ncv, M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
       CHECK(0 == ncvisual_rotate(ncv, M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
       CHECK(0 == ncvisual_rotate(ncv, M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
       CHECK(0 == ncvisual_rotate(ncv, M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
     }
     ncvisual_destroy(ncv);
@@ -231,11 +235,13 @@ TEST_CASE("Rotate") {
     auto ncv = ncvisual_from_rgba(rgba.data(), height, width * 4, width);
     REQUIRE(ncv);
     ncvisual_options opts{};
-    auto rendered = ncvisual_render(nc_, ncv, &opts);
+    opts.n = n_;
+    opts.flags = NCVISUAL_OPTION_CHILDPLANE;
+    auto rendered = ncvisual_blit(nc_, ncv, &opts);
     REQUIRE(rendered);
-    int pxdimy, pxdimx;
+    unsigned pxdimy, pxdimx;
     uint32_t* rgbaret = ncplane_as_rgba(rendered, NCBLIT_2x1,
-                                        0, 0, -1, -1, &pxdimy, &pxdimx);
+                                        0, 0, 0, 0, &pxdimy, &pxdimx);
     REQUIRE(rgbaret);
     if(height % 2){
       ++height;
@@ -267,16 +273,16 @@ TEST_CASE("Rotate") {
       // FIXME check pixels after all rotations
       opts.n = rendered;
       CHECK(0 == ncvisual_rotate(ncv, -M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
       CHECK(0 == ncvisual_rotate(ncv, -M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
       CHECK(0 == ncvisual_rotate(ncv, -M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
       CHECK(0 == ncvisual_rotate(ncv, -M_PI / 2));
-      CHECK(ncvisual_render(nc_, ncv, &opts));
+      CHECK(ncvisual_blit(nc_, ncv, &opts));
       CHECK(0 == notcurses_render(nc_));
     }
     ncvisual_destroy(ncv);
