@@ -1,7 +1,6 @@
 #include "main.h"
 #include <array>
 #include <cstdlib>
-#include "internal.h"
 
 TEST_CASE("Scrolling") {
   auto nc_ = testing_notcurses();
@@ -33,7 +32,7 @@ TEST_CASE("Scrolling") {
     CHECK(!ncplane_set_scrolling(n, false));
     // try to write 40 EGCs; it ought fail after 20
     CHECK(-20 == ncplane_putstr(n, "0123456789012345678901234567890123456789"));
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n, &y, &x);
     CHECK(0 == y);
     CHECK(20 == x);
@@ -82,7 +81,7 @@ TEST_CASE("Scrolling") {
     struct ncplane* n = ncplane_create(n_, &nopts);
     REQUIRE(n);
     CHECK(20 == ncplane_putstr(n, "01234567890123456789"));
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n, &y, &x);
     CHECK(0 == y);
     CHECK(20 == x);
@@ -120,7 +119,7 @@ TEST_CASE("Scrolling") {
         CHECK(0 > ret);
       }
     }
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n, &y, &x);
     CHECK(0 == y);
     CHECK(20 == x);
@@ -151,9 +150,9 @@ TEST_CASE("Scrolling") {
     REQUIRE(n);
     // verify that the new plane was started without scrolling
     CHECK(!ncplane_set_scrolling(n, true));
-    nccell ul = CELL_TRIVIAL_INITIALIZER, ur = CELL_TRIVIAL_INITIALIZER;
-    nccell dl = CELL_TRIVIAL_INITIALIZER, dr = CELL_TRIVIAL_INITIALIZER;
-    nccell hl = CELL_TRIVIAL_INITIALIZER, vl = CELL_TRIVIAL_INITIALIZER;
+    nccell ul = NCCELL_TRIVIAL_INITIALIZER, ur = NCCELL_TRIVIAL_INITIALIZER;
+    nccell dl = NCCELL_TRIVIAL_INITIALIZER, dr = NCCELL_TRIVIAL_INITIALIZER;
+    nccell hl = NCCELL_TRIVIAL_INITIALIZER, vl = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 == nccells_double_box(n, 0, 0, &ul, &ur, &dl, &dr, &hl, &vl));
     CHECK(0 > ncplane_box_sized(n, &ul, &ur, &dl, &dr, &hl, &vl, 2, 25, 0));
     CHECK(0 > ncplane_box_sized(n, &ul, &ur, &dl, &dr, &hl, &vl, 2, 21, 0));
@@ -178,7 +177,7 @@ TEST_CASE("Scrolling") {
     CHECK(!ncplane_set_scrolling(n, true));
     CHECK(40 == ncplane_putstr(n, out)); // fill up the plane w/ numbers
     CHECK(0 == notcurses_render(nc_));
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n, &y, &x);
     CHECK(1 == y);
     CHECK(20 == x);
@@ -256,6 +255,60 @@ TEST_CASE("Scrolling") {
     // FIXME
   }
 
+  // ensure that bound planes are scrolled along with us
+  SUBCASE("BoundScroll") {
+    int starty = 4;
+    struct ncplane_options nopts = {
+      .y = starty,
+      .x = 4,
+      .rows = 2,
+      .cols = 20,
+      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
+      .margin_b = 0, .margin_r = 0,
+    };
+    ncplane_set_scrolling(n_, true);
+    auto np = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != np);
+    nccell c = NCCELL_INITIALIZER('a', 0, NCCHANNELS_INITIALIZER(0xbb, 0, 0xbb, 0, 0, 0));
+    CHECK(0 < ncplane_polyfill_yx(np, 0, 0, &c));
+    CHECK(0 == ncplane_cursor_move_yx(n_, ncplane_dim_y(n_) - 1, 0));
+    CHECK(0 == notcurses_render(nc_));
+    for(unsigned i = 0 ; i < ncplane_dim_y(np) + starty ; ++i){
+      CHECK(starty - i == ncplane_y(np));
+      CHECK(1 == ncplane_putchar(n_, '\n'));
+      CHECK(0 == notcurses_render(nc_));
+    }
+    CHECK(0 == ncplane_destroy(np));
+  }
+
+  // ensure that bound planes are *not* scrolled along with us when the
+  // NCPLANE_OPTION_FIXED flag is used
+  SUBCASE("BoundScrollFixed") {
+    int starty = 4;
+    struct ncplane_options nopts = {
+      .y = starty,
+      .x = 4,
+      .rows = 2,
+      .cols = 20,
+      .userptr = nullptr, .name = nullptr, .resizecb = nullptr,
+      .flags = NCPLANE_OPTION_FIXED,
+      .margin_b = 0, .margin_r = 0,
+    };
+    ncplane_set_scrolling(n_, true);
+    auto np = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != np);
+    nccell c = NCCELL_INITIALIZER('a', 0, NCCHANNELS_INITIALIZER(0xbb, 0, 0xbb, 0, 0, 0));
+    CHECK(0 < ncplane_polyfill_yx(np, 0, 0, &c));
+    CHECK(0 == ncplane_cursor_move_yx(n_, ncplane_dim_y(n_) - 1, 0));
+    CHECK(0 == notcurses_render(nc_));
+    for(unsigned i = 0 ; i < ncplane_dim_y(np) + starty ; ++i){
+      CHECK(starty == ncplane_y(np));
+      CHECK(1 == ncplane_putchar(n_, '\n'));
+      CHECK(0 == notcurses_render(nc_));
+    }
+    CHECK(0 == ncplane_destroy(np));
+  }
+
   CHECK(0 == notcurses_stop(nc_));
 
 }
