@@ -11,24 +11,57 @@ TEST_CASE("Selectors") {
   REQUIRE(n_);
   REQUIRE(0 == ncplane_cursor_move_yx(n_, 0, 0));
 
+  // selector can't be bound to the standard plane
+  SUBCASE("RefuseStandardPlane") {
+    ncselector_options s{};
+    struct ncselector* ns = ncselector_create(n_, &s);
+    REQUIRE(nullptr == ns);
+  }
+
+  // create a selector, but don't explicitly destroy it, thus testing the
+  // context shutdown cleanup path
+  SUBCASE("ImplicitDestroy") {
+    ncselector_options s{};
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
+    struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
+    struct ncselector* ns = ncselector_create(n, &s);
+    REQUIRE(ns);
+    CHECK(0 == notcurses_render(nc_));
+  }
+
+  // now do the same, but with a plane we have created.
+  SUBCASE("RefuseBoundCreatedPlane") {
+    struct ncplane_options nopts{};
+    nopts.rows = ncplane_dim_y(n_);
+    nopts.cols = ncplane_dim_x(n_);
+    auto ncp = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != ncp);
+    ncselector_options s{};
+    struct ncselector* ns = ncselector_create(ncp, &s);
+    REQUIRE(ns);
+    CHECK(0 == notcurses_render(nc_));
+    struct ncselector* fail = ncselector_create(ncp, &s);
+    CHECK(nullptr == fail);
+    CHECK(0 == notcurses_render(nc_));
+  }
+
   SUBCASE("EmptySelector") {
     struct ncselector_options opts{};
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
     CHECK(nullptr == ncselector_selected(ncs));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(4 == dimy);
     CHECK(5 == dimx);
@@ -37,22 +70,19 @@ TEST_CASE("Selectors") {
 
   SUBCASE("TitledSelector") {
     struct ncselector_options opts{};
-    opts.title = strdup("hey hey whaddya say");
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    auto title = "hey hey whaddya say";
+    opts.title = title;
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(6 == dimy);
     CHECK(strlen(opts.title) + 4 == dimx);
@@ -61,22 +91,19 @@ TEST_CASE("Selectors") {
 
   SUBCASE("SecondarySelector") {
     struct ncselector_options opts{};
-    opts.secondary = strdup("this is not a title, but it's not *not* a title");
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    auto secondary = "this is not a title, but it's not *not* a title";
+    opts.secondary = secondary;
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(4 == dimy);
     CHECK(strlen(opts.secondary) + 2 == dimx);
@@ -85,22 +112,19 @@ TEST_CASE("Selectors") {
 
   SUBCASE("FooterSelector") {
     struct ncselector_options opts{};
-    opts.footer = strdup("i am a lone footer, little old footer");
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    auto foot = "i am a lone footer, little old footer";
+    opts.footer = foot;
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(4 == dimy);
     CHECK(strlen(opts.footer) + 2 == dimx);
@@ -109,28 +133,24 @@ TEST_CASE("Selectors") {
 
   SUBCASE("PopulatedSelector") {
     ncselector_item items[] = {
-      { strdup("op1"), strdup("this is option 1"), 0, 0, },
-      { strdup("2ndop"), strdup("this is option #2"), 0, 0, },
-      { strdup("tres"), strdup("option the third"), 0, 0, },
-      { NULL, NULL, 0, 0, },
+      { "op1", "this is option 1", },
+      { "2ndop", "this is option #2", },
+      { "tres", "option the third", },
+      { nullptr, nullptr, },
     };
     struct ncselector_options opts{};
     opts.items = items;
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(7 == dimy);
     CHECK(15 < dimx);
@@ -139,15 +159,11 @@ TEST_CASE("Selectors") {
 
   SUBCASE("EmptySelectorMovement") {
     struct ncselector_options opts{};
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
@@ -164,22 +180,18 @@ TEST_CASE("Selectors") {
 
   SUBCASE("SelectorMovement") {
     ncselector_item items[] = {
-      { strdup("op1"), strdup("this is option 1"), 0, 0, },
-      { strdup("2ndop"), strdup("this is option #2"), 0, 0, },
-      { strdup("tres"), strdup("option the third"), 0, 0, },
-      { NULL, NULL, 0, 0, },
+      { "op1", "this is option 1", },
+      { "2ndop", "this is option #2", },
+      { "tres", "option the third", },
+      { nullptr, nullptr, },
     };
     struct ncselector_options opts{};
     opts.items = items;
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     auto sel = ncselector_selected(ncs);
@@ -210,23 +222,19 @@ TEST_CASE("Selectors") {
   // Provide three items, limited to 1 shown at a time
   SUBCASE("ScrollingSelectorOne") {
     ncselector_item items[] = {
-      { strdup("op1"), strdup("this is option 1"), 0, 0, },
-      { strdup("2ndop"), strdup("this is option #2"), 0, 0, },
-      { strdup("tres"), strdup("option the third"), 0, 0, },
-      { NULL, NULL, 0, 0, },
+      { "op1", "this is option 1", },
+      { "2ndop", "this is option #2", },
+      { "tres", "option the third", },
+      { nullptr, nullptr, },
     };
     struct ncselector_options opts{};
     opts.maxdisplay = 1;
     opts.items = items;
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
@@ -253,7 +261,7 @@ TEST_CASE("Selectors") {
     CHECK(0 == strcmp(sel, items[0].option));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(5 == dimy);
     ncselector_destroy(ncs, nullptr);
@@ -262,23 +270,19 @@ TEST_CASE("Selectors") {
   // Provide three items, limited to 2 shown at a time
   SUBCASE("ScrollingSelectorTwo") {
     ncselector_item items[] = {
-      { strdup("op1"), strdup("this is option 1"), 0, 0, },
-      { strdup("2ndop"), strdup("this is option #2"), 0, 0, },
-      { strdup("tres"), strdup("option the third"), 0, 0, },
-      { NULL, NULL, 0, 0, },
+      { "op1", "this is option 1", },
+      { "2ndop", "this is option #2", },
+      { "tres", "option the third", },
+      { nullptr, nullptr, },
     };
     struct ncselector_options opts{};
     opts.maxdisplay = 2;
     opts.items = items;
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = 1,
-      .cols = 1,
-      .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = 1;
+    nopts.cols = 1;
     struct ncplane* n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
     struct ncselector* ncs = ncselector_create(n, &opts);
     REQUIRE(nullptr != ncs);
     CHECK(0 == notcurses_render(nc_));
@@ -304,7 +308,7 @@ TEST_CASE("Selectors") {
     CHECK(0 == strcmp(sel, items[0].option));
     struct ncplane* ncsp = ncselector_plane(ncs);
     REQUIRE(nullptr != ncsp);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncsp, &dimy, &dimx);
     CHECK(6 == dimy);
     ncselector_destroy(ncs, nullptr);
