@@ -1,5 +1,4 @@
 #include "main.h"
-#include "internal.h"
 
 TEST_CASE("TextLayout") {
   auto nc_ = testing_notcurses();
@@ -11,6 +10,31 @@ TEST_CASE("TextLayout") {
 
   const char str[] = "this is going to be broken up";
 
+  SUBCASE("LayoutUnaligned") {
+    struct ncplane_options nopts = {
+      .y = 0,
+      .x = 0,
+      .rows = 2,
+      .cols = 20,
+      .userptr = nullptr,
+      .name = nullptr,
+      .resizecb = nullptr,
+      .flags = 0,
+      .margin_b = 0, .margin_r = 0,
+    };
+    auto sp = ncplane_create(n_, &nopts);
+    REQUIRE(sp);
+    size_t bytes;
+    CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_UNALIGNED, str, &bytes));
+    CHECK(0 == notcurses_render(nc_));
+    CHECK(bytes == strlen(str));
+    char* line = ncplane_contents(sp, 0, 0, 2, 20);
+    REQUIRE(line);
+    CHECK(0 == strcmp(line, "this is going to be broken up"));
+    free(line);
+    CHECK(0 == ncplane_destroy(sp));
+  }
+
   SUBCASE("LayoutLeft") {
     struct ncplane_options nopts = {
       .y = 0,
@@ -33,7 +57,7 @@ TEST_CASE("TextLayout") {
     REQUIRE(line);
     CHECK(0 == strcmp(line, "this is going to be broken up"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   SUBCASE("LayoutRight") {
@@ -58,7 +82,7 @@ TEST_CASE("TextLayout") {
     REQUIRE(line);
     CHECK(0 == strcmp(line, "this is going to be broken up"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   SUBCASE("LayoutCenter") {
@@ -83,7 +107,7 @@ TEST_CASE("TextLayout") {
     REQUIRE(line);
     CHECK(0 == strcmp(line, "this is going to be broken up"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // lay out text where a word ends on the boundary
@@ -106,11 +130,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_CENTER, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "my nuclear arms"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // lay out text where a word crosses the boundary
@@ -133,11 +157,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_CENTER, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "my grasping arms"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // ensure we're honoring newlines
@@ -160,11 +184,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "abcde")); // FIXME should have newlines
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // ensure we're honoring newlines at the start/end of rows
@@ -187,11 +211,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "abcdefgh"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // lay out text where a wide word crosses the boundary
@@ -215,11 +239,11 @@ TEST_CASE("TextLayout") {
       CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_CENTER, boundstr, &bytes));
       CHECK(0 == notcurses_render(nc_));
       CHECK(bytes == strlen(boundstr));
-      char* line = ncplane_contents(sp, 0, 0, -1, -1);
+      char* line = ncplane_contents(sp, 0, 0, 0, 0);
       REQUIRE(line);
       CHECK(0 == strcmp(line, boundstr));
       free(line);
-      ncplane_destroy(sp);
+      CHECK(0 == ncplane_destroy(sp));
     }
   }
 
@@ -244,11 +268,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_CENTER, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "my thermonuclear arms"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // a long word (one requiring a split no matter what) ought not force the
@@ -273,11 +297,11 @@ TEST_CASE("TextLayout") {
       CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_CENTER, boundstr, &bytes));
       CHECK(0 == notcurses_render(nc_));
       CHECK(bytes == strlen(boundstr));
-      char* line = ncplane_contents(sp, 0, 0, -1, -1);
+      char* line = ncplane_contents(sp, 0, 0, 0, 0);
       REQUIRE(line);
       CHECK(0 == strcmp(line, "1 我能吞下玻璃"));
       free(line);
-      ncplane_destroy(sp);
+      CHECK(0 == ncplane_destroy(sp));
     }
   }
 
@@ -300,11 +324,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_CENTER, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "    my thermonuclear arms"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // create a plane of two rows, and fill exactly one with one word
@@ -327,11 +351,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "quarkgluonfart "));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // create a plane of two rows, and fill exactly one with words
@@ -354,11 +378,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "quark gluon fart "));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // create a plane of two rows, and exactly fill the first line
@@ -381,11 +405,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "quantum balls"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // create a plane of three rows, and exactly fill two with regular ol' words
@@ -408,11 +432,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "quantum balls scratchy no?! "));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // create a plane of three rows, and exactly fill two, with no spaces
@@ -435,11 +459,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "0123456789AB"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   // create a plane of three rows, and exactly fill two with wide chars
@@ -463,11 +487,11 @@ TEST_CASE("TextLayout") {
       CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
       CHECK(0 == notcurses_render(nc_));
       CHECK(bytes == strlen(boundstr));
-      char* line = ncplane_contents(sp, 0, 0, -1, -1);
+      char* line = ncplane_contents(sp, 0, 0, 0, 0);
       REQUIRE(line);
       CHECK(0 == strcmp(line, "我能吞 下玻璃 "));
       free(line);
-      ncplane_destroy(sp);
+      CHECK(0 == ncplane_destroy(sp));
     }
   }
 
@@ -493,11 +517,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 > res);
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes < strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "quantum balls scratchy no?! "));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   SUBCASE("LayoutLongScroll") {
@@ -520,11 +544,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, boundstr, &bytes));
     CHECK(0 == notcurses_render(nc_));
     CHECK(bytes == strlen(boundstr));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "scratchy?! true! arrrrp"));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   SUBCASE("LayoutLongLines") {
@@ -554,11 +578,11 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, text, &bytes));
     CHECK(bytes == strlen(text));
     CHECK(0 == notcurses_render(nc_));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
     CHECK(0 == strcmp(line, "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Proin ornare neque ac ipsum viverra, vestibulum hendrerit leo consequat. Integer velit, pharetra sed nisl quis, porttitor ornare purus. Cras ac sollicitudin dolor, eget elementum dolor. Quisque lobortis sagittis."));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   SUBCASE("LayoutZooText") {
@@ -571,30 +595,22 @@ TEST_CASE("TextLayout") {
       "NCFdplane streams a file descriptor, while NCSubproc spawns a subprocess and streams its output. "
       "A variety of plots are supported, and menus can be placed along the top and/or bottom of any plane.\n\n"
       "Widgets can be controlled with the keyboard and/or mouse. They are implemented atop ncplanes, and these planes can be manipulated like all others.";
-    struct ncplane_options nopts = {
-      .y = 0,
-      .x = 0,
-      .rows = READER_ROWS,
-      .cols = READER_COLS,
-      .userptr = nullptr,
-      .name = nullptr,
-      .resizecb = nullptr,
-      .flags = 0,
-      .margin_b = 0, .margin_r = 0,
-    };
+    struct ncplane_options nopts{};
+    nopts.rows = READER_ROWS;
+    nopts.cols = READER_COLS;
+    nopts.flags = NCPLANE_OPTION_VSCROLL;
     auto sp = ncplane_create(n_, &nopts);
     REQUIRE(sp);
-    ncplane_set_scrolling(sp, true);
     size_t bytes;
     ncplane_home(sp);
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, text, &bytes));
     CHECK(bytes == strlen(text));
     CHECK(0 == notcurses_render(nc_));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
-    CHECK(0 == strcmp(line, "to be selected from a list of n items. NCFdplane streams a file descriptor, while NCSubproc spawns a subprocess and streams its output. A variety of plots are supported, and menus can be placed along the top and/or bottom of any plane.Widgets can be controlled with the keyboard and/or mouse. They are implemented atop ncplanes, and these planes can be manipulated like all others."));
+    CHECK(0 == strcmp(line, "to be selected from a list of n items. NCFdplane streams a file descriptor, while NCSubproc spawns a subprocess and streams its output. A variety of plots are supported, and menus can be placed along the top and/or bottom of any plane.\nWidgets can be controlled with the keyboard and/or mouse. They are implemented atop ncplanes, and these planes can be manipulated like all others."));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
   }
 
   SUBCASE("LayoutZooTextNoScroll") {
@@ -627,11 +643,78 @@ TEST_CASE("TextLayout") {
     CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, text, &bytes));
     CHECK(bytes == strlen(text));
     CHECK(0 == notcurses_render(nc_));
-    char* line = ncplane_contents(sp, 0, 0, -1, -1);
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
     REQUIRE(line);
-    CHECK(0 == strcmp(line, "Notcurses provides several widgets to quickly build vivid TUIs.This NCReader widget facilitates free-form text entry complete with readline-style bindings. NCSelector allows a single option to be selected from a list. NCMultiselector allows 0..n options to be selected from a list of n items. NCFdplane streams a file descriptor, while NCSubproc spawns a subprocess and streams its output. A variety of plots are supported, and menus can be placed along the top and/or bottom of any plane.Widgets can be controlled with the keyboard and/or mouse. They are implemented atop ncplanes, and these planes can be manipulated like all others."));
+    CHECK(0 == strcmp(line, "Notcurses provides several widgets to quickly build vivid TUIs."
+                            "This NCReader widget facilitates free-form text entry complete "
+                            "with readline-style bindings. NCSelector allows a single option "
+                            "to be selected from a list. NCMultiselector allows 0..n options "
+                            "to be selected from a list of n items. NCFdplane streams a file "
+                            "descriptor, while NCSubproc spawns a subprocess and streams its "
+                            "output. A variety of plots are supported, and menus can be placed "
+                            "along the top and/or bottom of any plane.Widgets can be controlled "
+                            "with the keyboard and/or mouse. They are implemented atop ncplanes, "
+                            "and these planes can be manipulated like all others."));
     free(line);
-    ncplane_destroy(sp);
+    CHECK(0 == ncplane_destroy(sp));
+  }
+
+  // there shouldn't be any leftover material from lines scrolled off the plane
+  SUBCASE("ScrollClearsEnd") {
+    const char text[] = "1234\n12\n54";
+    struct ncplane_options nopts{};
+    nopts.rows = 2;
+    nopts.cols = 5;
+    auto sp = ncplane_create(n_, &nopts);
+    REQUIRE(sp);
+    ncplane_set_scrolling(sp, true);
+    size_t bytes;
+    CHECK(0 < ncplane_puttext(sp, 0, NCALIGN_LEFT, text, &bytes));
+    CHECK(bytes == strlen(text));
+    CHECK(0 == notcurses_render(nc_));
+    char* line = ncplane_contents(sp, 0, 0, 0, 0);
+    REQUIRE(line);
+    CHECK(0 == strcmp(line, "1254"));
+    free(line);
+    CHECK(0 == ncplane_destroy(sp));
+  }
+
+  // test that multiple new lines are treated as such, both on the plane
+  // originally, and in any autogrown region.
+  SUBCASE("MultipleNewlines") {
+    struct ncplane_options nopts{};
+    nopts.rows = 3;
+    nopts.cols = 10;
+    nopts.flags = NCPLANE_OPTION_VSCROLL | NCPLANE_OPTION_AUTOGROW;
+    auto nn = ncplane_create(n_, &nopts);
+    REQUIRE(nn);
+    size_t b;
+    CHECK(0 == ncplane_puttext(nn, -1, NCALIGN_LEFT, "\n\n", &b));
+    unsigned y, x;
+    ncplane_cursor_yx(nn, &y, &x);
+    CHECK(2 == y);
+    CHECK(0 == x);
+    CHECK(3 == ncplane_puttext(nn, -1, NCALIGN_LEFT, "erp", &b));
+    ncplane_cursor_yx(nn, &y, &x);
+    CHECK(2 == y);
+    CHECK(3 == x);
+    CHECK(0 == notcurses_render(nc_));
+    CHECK(0 == ncplane_puttext(nn, -1, NCALIGN_LEFT, "\n", &b));
+    ncplane_cursor_yx(nn, &y, &x);
+    CHECK(3 == y);
+    CHECK(0 == x);
+    CHECK(0 == notcurses_render(nc_));
+    CHECK(0 == ncplane_puttext(nn, -1, NCALIGN_LEFT, "\n\n", &b));
+    ncplane_cursor_yx(nn, &y, &x);
+    CHECK(5 == y);
+    CHECK(0 == x);
+    CHECK(0 == notcurses_render(nc_));
+    CHECK(3 == ncplane_puttext(nn, -1, NCALIGN_LEFT, "erp\n", &b));
+    ncplane_cursor_yx(nn, &y, &x);
+    CHECK(6 == y);
+    CHECK(0 == x);
+    CHECK(0 == notcurses_render(nc_));
+    CHECK(0 == ncplane_destroy(nn));
   }
 
   CHECK(0 == notcurses_stop(nc_));
