@@ -1,8 +1,25 @@
 #include "main.h"
-#include "visual-details.h"
+#include "lib/visual-details.h"
 #include <vector>
 #include <cmath>
 
+// verify results for extrinsic geometries with NULL or default vopts
+void default_visual_extrinsics(const notcurses* nc, const ncvgeom& g) {
+  CHECK(0 == g.pixy);
+  CHECK(0 == g.pixx);
+  CHECK(nc->tcache.cellpxy == g.cdimy);
+  CHECK(nc->tcache.cellpxx == g.cdimx);
+  CHECK(1 <= g.scaley);
+  CHECK(1 <= g.scalex);
+  CHECK(0 == g.rpixy);
+  CHECK(0 == g.rpixx);
+  CHECK(0 <= g.maxpixely);
+  CHECK(0 <= g.maxpixelx);
+  // we never use pixel by default, and must not revolve to default
+  CHECK(NCBLIT_PIXEL != g.blitter);
+  CHECK(NCBLIT_DEFAULT != g.blitter);
+}
+
 TEST_CASE("Visual") {
   auto nc_ = testing_notcurses();
   REQUIRE(nullptr != nc_);
@@ -27,6 +44,142 @@ TEST_CASE("Visual") {
         CHECK(ncpixel_a(p) == alpha);
       }
     }
+    ncvisual_destroy(ncv);
+  }
+
+  // ncvisual_geom() with a NULL nc
+  SUBCASE("VisualIntrinsicGeometry") {
+    std::vector<uint32_t> v(20, 0xfffffffflu);
+    auto ncv = ncvisual_from_rgba(v.data(), 2, 10 * sizeof(decltype(v)::value_type), 10);
+    REQUIRE(nullptr != ncv);
+    ncvgeom g{};
+    CHECK(0 == ncvisual_geom(nullptr, ncv, nullptr, &g));
+    ncvisual_destroy(ncv);
+    CHECK(2 == g.pixy);
+    CHECK(10 == g.pixx);
+    CHECK(0 == g.cdimy);
+    CHECK(0 == g.cdimx);
+    CHECK(0 == g.rpixy);
+    CHECK(0 == g.rpixx);
+    CHECK(0 == g.scaley);
+    CHECK(0 == g.scalex);
+    CHECK(0 == g.maxpixely);
+    CHECK(0 == g.maxpixelx);
+    CHECK(NCBLIT_DEFAULT == g.blitter);
+  }
+
+  // ncvisual_geom() with a NULL ncvisual and NULL visual_options
+  SUBCASE("VisualExtrinsicGeometryNULL") {
+    ncvgeom g{};
+    CHECK(0 == ncvisual_geom(nc_, nullptr, nullptr, &g));
+    default_visual_extrinsics(nc_, g);
+  }
+
+  // ncvisual_geom() with a NULL ncvisual and default visual_options
+  SUBCASE("VisualExtrinsicGeometryDefault") {
+    ncvgeom g{};
+    struct ncvisual_options vopts{};
+    CHECK(0 == ncvisual_geom(nc_, nullptr, &vopts, &g));
+    default_visual_extrinsics(nc_, g);
+  }
+
+  // ncvisual_geom() with a NULL ncvisual and NCBLIT_PIXEL requested
+  SUBCASE("VisualExtrinsicGeometryPixel") {
+    ncvgeom g{};
+    struct ncvisual_options vopts{};
+    vopts.blitter = NCBLIT_PIXEL;
+    CHECK(0 == ncvisual_geom(nc_, nullptr, &vopts, &g));
+    CHECK(0 == g.pixy);
+    CHECK(0 == g.pixx);
+    CHECK(nc_->tcache.cellpxy == g.cdimy);
+    CHECK(nc_->tcache.cellpxx == g.cdimx);
+    if(notcurses_canpixel(nc_)){
+      CHECK(g.cdimy == g.scaley);
+      CHECK(g.cdimx == g.scalex);
+      CHECK(nc_->tcache.sixel_maxy <= g.maxpixely);
+      CHECK(nc_->tcache.sixel_maxx <= g.maxpixelx);
+    }else{
+      CHECK(1 <= g.scaley);
+      CHECK(1 <= g.scalex);
+      CHECK(0 == g.maxpixely);
+      CHECK(0 == g.maxpixelx);
+    }
+    CHECK(0 == g.rpixy);
+    CHECK(0 == g.rpixx);
+    CHECK(NCBLIT_DEFAULT != g.blitter); // we must not revolve to default
+  }
+
+  // build a simple ncvisual and check the calculated geometries for 1x1
+  // cell blitting in the absence of scaling
+  SUBCASE("VisualCellGeometryNoScaling") {
+    std::vector<uint32_t> v(80, 0xfffffffflu);
+    auto ncv = ncvisual_from_rgba(v.data(), 8, 10 * sizeof(decltype(v)::value_type), 10);
+    REQUIRE(nullptr != ncv);
+    struct ncvisual_options vopts{};
+    ncvgeom g{};
+    vopts.blitter = NCBLIT_1x1;
+    CHECK(0 == ncvisual_geom(nc_, ncv, &vopts, &g));
+    ncvisual_destroy(ncv);
+    CHECK(8 == g.pixy);
+    CHECK(10 == g.pixx);
+    CHECK(1 == g.scaley);
+    CHECK(1 == g.scalex);
+    CHECK(8 == g.rpixy);
+    CHECK(10 == g.rpixx);
+    CHECK(8 == g.rcelly);
+    CHECK(10 == g.rcellx);
+    CHECK(NCBLIT_1x1 == g.blitter);
+  }
+
+  // build a square ncvisual and check the calculated geometries for 1x1
+  // cell blitting with scaling
+  SUBCASE("VisualCellGeometryScaling") {
+    std::vector<uint32_t> v(100, 0xfffffffflu);
+    auto ncv = ncvisual_from_rgba(v.data(), 10, 10 * sizeof(decltype(v)::value_type), 10);
+    REQUIRE(nullptr != ncv);
+    struct ncvisual_options vopts{};
+    ncvgeom g{};
+    vopts.blitter = NCBLIT_1x1;
+    vopts.scaling = NCSCALE_SCALE;
+    CHECK(0 == ncvisual_geom(nc_, ncv, &vopts, &g));
+    ncvisual_destroy(ncv);
+    unsigned dimy, dimx;
+    ncplane_dim_yx(n_, &dimy, &dimx);
+    unsigned mindim = dimy < dimx ? dimy : dimx;
+    CHECK(10 == g.pixy);
+    CHECK(10 == g.pixx);
+    CHECK(1 == g.scaley);
+    CHECK(1 == g.scalex);
+    CHECK(mindim == g.rpixy);
+    CHECK(mindim == g.rpixx);
+    CHECK(mindim == g.rcelly);
+    CHECK(mindim == g.rcellx);
+    CHECK(NCBLIT_1x1 == g.blitter);
+  }
+
+  // build a square ncvisual and check the calculated geometries for 1x1
+  // cell blitting with stretching
+  SUBCASE("VisualCellGeometryStretching") {
+    std::vector<uint32_t> v(100, 0xfffffffflu);
+    auto ncv = ncvisual_from_rgba(v.data(), 10, 10 * sizeof(decltype(v)::value_type), 10);
+    REQUIRE(nullptr != ncv);
+    struct ncvisual_options vopts{};
+    ncvgeom g{};
+    vopts.blitter = NCBLIT_1x1;
+    vopts.scaling = NCSCALE_STRETCH;
+    CHECK(0 == ncvisual_geom(nc_, ncv, &vopts, &g));
+    ncvisual_destroy(ncv);
+    unsigned dimy, dimx;
+    ncplane_dim_yx(n_, &dimy, &dimx);
+    CHECK(10 == g.pixy);
+    CHECK(10 == g.pixx);
+    CHECK(1 == g.scaley);
+    CHECK(1 == g.scalex);
+    CHECK(dimy == g.rpixy);
+    CHECK(dimx == g.rpixx);
+    CHECK(dimy == g.rcelly);
+    CHECK(dimx == g.rcellx);
+    CHECK(NCBLIT_1x1 == g.blitter);
   }
 
   // check that we properly populate RGB + A -> RGBA from 35x4 (see #1806)
@@ -45,6 +198,7 @@ TEST_CASE("Visual") {
         CHECK(ncpixel_a(p) == alpha);
       }
     }
+    ncvisual_destroy(ncv);
   }
 
   // check that we properly populate RGBx + A -> RGBA
@@ -63,6 +217,7 @@ TEST_CASE("Visual") {
         CHECK(ncpixel_a(p) == alpha);
       }
     }
+    ncvisual_destroy(ncv);
   }
 
   // resize followed by rotate, see #1800
@@ -74,18 +229,20 @@ TEST_CASE("Visual") {
     struct ncvisual_options vopts{};
     vopts.x = NCALIGN_CENTER;
     vopts.y = NCALIGN_CENTER;
-    vopts.flags |= NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_VERALIGNED;
-    auto p = ncvisual_render(nc_, ncv, &vopts);
+    vopts.n = n_;
+    vopts.flags |= NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_VERALIGNED
+                    | NCVISUAL_OPTION_CHILDPLANE;
+    auto p = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != p);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(p));
     CHECK(0 == ncvisual_resize(ncv, 20, 20));
-    p = ncvisual_render(nc_, ncv, &vopts);
+    p = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != p);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(p));
     CHECK(0 == ncvisual_rotate(ncv, M_PI / 2));
-    p = ncvisual_render(nc_, ncv, &vopts);
+    p = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != p);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_destroy(p));
@@ -96,7 +253,7 @@ TEST_CASE("Visual") {
   SUBCASE("VisualAligned") {
     const uint32_t pixels[4] = { htole(0xffff0000), htole(0xff00ff00), htole(0xff0000ff), htole(0xffffffff) };
     ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_NONE,
       .y = 0,
       .x = NCALIGN_LEFT,
@@ -105,12 +262,13 @@ TEST_CASE("Visual") {
       .leny = 2,
       .lenx = 2,
       .blitter = NCBLIT_1x1,
-      .flags = NCVISUAL_OPTION_HORALIGNED,
+      .flags = NCVISUAL_OPTION_HORALIGNED | NCVISUAL_OPTION_CHILDPLANE,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     auto ncv = ncvisual_from_rgba(pixels, 2, 2 * sizeof(*pixels), 2);
     REQUIRE(nullptr != ncv);
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     CHECK(0 == notcurses_render(nc_));
     ncvisual_destroy(ncv);
@@ -125,15 +283,17 @@ TEST_CASE("Visual") {
     auto ncv = ncvisual_from_rgba(v.data(), y, sizeof(decltype(v)::value_type) * x, x);
     REQUIRE(nullptr != ncv);
     struct ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_NONE,
       .y = 0, .x = 0,
       .begy = 0, .begx = 0,
       .leny = 5, .lenx = 8,
       .blitter = NCBLIT_1x1,
-      .flags = 0, .transcolor = 0,
+      .flags = NCVISUAL_OPTION_CHILDPLANE,
+      .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     CHECK(5 == ncplane_dim_y(n));
     CHECK(8 == ncplane_dim_x(n));
@@ -144,28 +304,24 @@ TEST_CASE("Visual") {
   // ensure that NCSCALE_STRETCH gives us a full plane, and that we write
   // everywhere within that plane
   SUBCASE("Stretch") {
-    std::vector<uint32_t> v(1, htole(0xe61c28ff));
-    int dimy, dimx;
+    std::vector<uint32_t> v(1, htole(0xff1c28ff));
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     auto ncv = ncvisual_from_rgba(v.data(), 1, sizeof(decltype(v)::value_type), 1);
     REQUIRE(nullptr != ncv);
-    struct ncvisual_options vopts = {
-      .n = nullptr,
-      .scaling = NCSCALE_STRETCH,
-      .y = 0, .x = 0,
-      .begy = 0, .begx = 0,
-      .leny = 0, .lenx = 0,
-      .blitter = NCBLIT_1x1,
-      .flags = 0, .transcolor = 0,
-    };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    struct ncvisual_options vopts{};
+    vopts.n = n_;
+    vopts.scaling = NCSCALE_STRETCH;
+    vopts.blitter = NCBLIT_1x1;
+    vopts.flags = NCVISUAL_OPTION_CHILDPLANE;
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     CHECK(0 == notcurses_render(nc_));
     REQUIRE(nullptr != n);
     CHECK(dimy == ncplane_dim_y(n));
     CHECK(dimx == ncplane_dim_x(n));
     ncvisual_destroy(ncv);
-    for(int y = 0 ; y < dimy ; ++y){
-      for(int x = 0 ; x < dimx ; ++x){
+    for(unsigned y = 0 ; y < dimy ; ++y){
+      for(unsigned x = 0 ; x < dimx ; ++x){
         uint16_t stylemask;
         uint64_t channels;
         auto c = ncplane_at_yx(n, y, x, &stylemask, &channels);
@@ -193,8 +349,9 @@ TEST_CASE("Visual") {
       .leny = 5, .lenx = 8,
       .blitter = NCBLIT_1x1,
       .flags = 0, .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto n = ncvisual_render(nc_, ncv, &vopts);
+    auto n = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(nullptr != n);
     CHECK(5 == ncplane_dim_y(n));
     CHECK(8 == ncplane_dim_x(n));
@@ -216,8 +373,9 @@ TEST_CASE("Visual") {
       .leny = 0, .lenx = 0,
       .blitter = NCBLIT_1x1,
       .flags = 0, .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
-    auto newn = ncvisual_render(nc_, ncv, &vopts);
+    auto newn = ncvisual_blit(nc_, ncv, &vopts);
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncvisual_resize_noninterpolative(ncv, ncv->pixy * 3, ncv->pixx * 3));
     CHECK(6 == ncv->pixy);
@@ -239,8 +397,8 @@ TEST_CASE("Visual") {
       }
     }
     REQUIRE(newn);
-    auto enewn = ncvisual_render(nc_, ncv, &vopts);
-    int newy, newx;
+    auto enewn = ncvisual_blit(nc_, ncv, &vopts);
+    unsigned newy, newx;
     ncplane_dim_yx(enewn, &newy, &newx);
     CHECK(6 == newy);
     CHECK(6 == newx);
@@ -251,7 +409,7 @@ TEST_CASE("Visual") {
   }
 
   SUBCASE("LoadRGBAFromMemory") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     // alpha, then b, g, r
     std::vector<uint32_t> rgba(dimx * dimy * 2, htole(0xff88bbccull));
@@ -260,10 +418,10 @@ TEST_CASE("Visual") {
     struct ncvisual_options opts{};
     opts.blitter = NCBLIT_1x1;
     opts.n = ncp_;
-    CHECK(ncp_ == ncvisual_render(nc_, ncv, &opts));
+    CHECK(ncp_ == ncvisual_blit(nc_, ncv, &opts));
     CHECK(0 == notcurses_render(nc_));
-    for(int y = 0 ; y < dimy ; ++y){
-      for(int x = 0 ; x < dimx ; ++x){
+    for(unsigned y = 0 ; y < dimy ; ++y){
+      for(unsigned x = 0 ; x < dimx ; ++x){
         uint16_t stylemask;
         uint64_t channels;
         auto c = ncplane_at_yx(ncp_, y, x, &stylemask, &channels);
@@ -278,7 +436,7 @@ TEST_CASE("Visual") {
   }
 
   SUBCASE("LoadBGRAFromMemory") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     // A should be at the highest memory address, which would be the most
     // significant byte on little-endian. then r, g, b.
@@ -288,10 +446,10 @@ TEST_CASE("Visual") {
     struct ncvisual_options opts{};
     opts.blitter = NCBLIT_1x1;
     opts.n = ncp_;
-    CHECK(nullptr != ncvisual_render(nc_, ncv, &opts));
+    CHECK(nullptr != ncvisual_blit(nc_, ncv, &opts));
     CHECK(0 == notcurses_render(nc_));
-    for(int y = 0 ; y < dimy ; ++y){
-      for(int x = 0 ; x < dimx ; ++x){
+    for(unsigned y = 0 ; y < dimy ; ++y){
+      for(unsigned x = 0 ; x < dimx ; ++x){
         uint16_t stylemask;
         uint64_t channels;
         auto c = ncplane_at_yx(ncp_, y, x, &stylemask, &channels);
@@ -328,7 +486,7 @@ TEST_CASE("Visual") {
       vopts.n = n_;
       vopts.blitter = NCBLIT_2x1;
       vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-      CHECK(n_ == ncvisual_render(nc_, ncv, &vopts));
+      CHECK(n_ == ncvisual_blit(nc_, ncv, &vopts));
       CHECK(0 == notcurses_render(nc_));
       for(int y = 0 ; y < DIMY / 2 ; ++y){
         for(int x = 0 ; x < DIMX ; ++x){
@@ -342,6 +500,7 @@ TEST_CASE("Visual") {
         }
       }
       delete[] rgba;
+      ncvisual_destroy(ncv);
     }
   }
 
@@ -368,7 +527,7 @@ TEST_CASE("Visual") {
       vopts.n = n_;
       vopts.blitter = NCBLIT_2x2;
       vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-      CHECK(n_ == ncvisual_render(nc_, ncv, &vopts));
+      CHECK(n_ == ncvisual_blit(nc_, ncv, &vopts));
       CHECK(0 == notcurses_render(nc_));
       for(int y = 0 ; y < DIMY / 2 ; ++y){
         for(int x = 0 ; x < DIMX / 2 ; ++x){
@@ -382,6 +541,7 @@ TEST_CASE("Visual") {
         }
       }
       delete[] rgba;
+      ncvisual_destroy(ncv);
     }
   }
 
@@ -426,7 +586,7 @@ TEST_CASE("Visual") {
       vopts.n = n_;
       vopts.blitter = NCBLIT_2x2;
       vopts.flags = NCVISUAL_OPTION_NODEGRADE;
-      CHECK(n_ == ncvisual_render(nc_, ncv, &vopts));
+      CHECK(n_ == ncvisual_blit(nc_, ncv, &vopts));
       CHECK(0 == notcurses_render(nc_));
       for(int y = 0 ; y < DIMY / 2 ; ++y){
         for(int x = 0 ; x < DIMX / 2 ; ++x){
@@ -456,6 +616,7 @@ TEST_CASE("Visual") {
         }
       }
       delete[] rgba;
+      ncvisual_destroy(ncv);
     }
   }
 
@@ -466,7 +627,7 @@ TEST_CASE("Visual") {
       auto ncv = ncvisual_from_rgba(pixels, 2, 2 * sizeof(*pixels), 2);
       REQUIRE(nullptr != ncv);
       struct ncvisual_options vopts = {
-        .n = nullptr,
+        .n = n_,
         .scaling = NCSCALE_NONE,
         .y = 0,
         .x = 0,
@@ -475,12 +636,13 @@ TEST_CASE("Visual") {
         .leny = 0,
         .lenx = 0,
         .blitter = NCBLIT_2x2,
-        .flags = 0,
+        .flags = NCVISUAL_OPTION_CHILDPLANE,
         .transcolor = 0,
+        .pxoffy = 0, .pxoffx = 0,
       };
-      auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+      auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
       REQUIRE(nullptr != ncvp);
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncvp, &dimy, &dimx);
       CHECK(1 == dimy);
       CHECK(1 == dimx);
@@ -510,7 +672,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -519,12 +681,13 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
-        int dimy, dimx;
+        unsigned dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
         CHECK(1 == dimy);
         CHECK(1 == dimx);
@@ -557,7 +720,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -566,12 +729,13 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
-        int dimy, dimx;
+        unsigned dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
         CHECK(1 == dimy);
         CHECK(1 == dimx);
@@ -609,7 +773,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -618,12 +782,13 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
-        int dimy, dimx;
+        unsigned dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
         CHECK(1 == dimy);
         CHECK(1 == dimx);
@@ -660,7 +825,7 @@ TEST_CASE("Visual") {
         auto ncv = ncvisual_from_rgba(pixels[i], 2, 2 * sizeof(**pixels), 2);
         REQUIRE(nullptr != ncv);
         struct ncvisual_options vopts = {
-          .n = nullptr,
+          .n = n_,
           .scaling = NCSCALE_NONE,
           .y = 0,
           .x = 0,
@@ -669,12 +834,13 @@ TEST_CASE("Visual") {
           .leny = 0,
           .lenx = 0,
           .blitter = NCBLIT_2x2,
-          .flags = 0,
+          .flags = NCVISUAL_OPTION_CHILDPLANE,
           .transcolor = 0,
+          .pxoffy = 0, .pxoffx = 0,
         };
-        auto ncvp = ncvisual_render(nc_, ncv, &vopts);
+        auto ncvp = ncvisual_blit(nc_, ncv, &vopts);
         REQUIRE(nullptr != ncvp);
-        int dimy, dimx;
+        unsigned dimy, dimx;
         ncplane_dim_yx(ncvp, &dimy, &dimx);
         CHECK(1 == dimy);
         CHECK(1 == dimx);
@@ -692,7 +858,7 @@ TEST_CASE("Visual") {
     }
   }
 
-  // test NCVISUAL_OPTIONS_CHILDPLANE + stretch + null alignment
+  // test NCVISUAL_OPTION_CHILDPLANE + stretch + null alignment
   SUBCASE("ImageChildScaling") {
     struct ncplane_options opts = {
       .y = 0, .x = 0,
@@ -707,15 +873,16 @@ TEST_CASE("Visual") {
     auto parent = ncplane_create(n_, &opts);
     REQUIRE(parent);
     struct ncvisual_options vopts = {
-      .n = nullptr,
+      .n = n_,
       .scaling = NCSCALE_NONE,
       .y = 0,
       .x = 0,
       .begy = 0, .begx = 0,
       .leny = 0, .lenx = 0,
       .blitter = NCBLIT_1x1,
-      .flags = 0,
+      .flags = NCVISUAL_OPTION_CHILDPLANE,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     const uint32_t pixels[16] = {
       htole(0xffffffff), htole(0xffffffff), htole(0xffc0ffff), htole(0xffffc0ff),
@@ -725,7 +892,7 @@ TEST_CASE("Visual") {
     };
     auto ncv = ncvisual_from_rgba(pixels, 4, 16, 4);
     REQUIRE(ncv);
-    auto child = ncvisual_render(nc_, ncv, &vopts);
+    auto child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(4 == ncplane_dim_y(child));
     CHECK(4 == ncplane_dim_x(child));
@@ -736,7 +903,7 @@ TEST_CASE("Visual") {
     vopts.n = parent,
     vopts.scaling = NCSCALE_STRETCH,
     vopts.flags = NCVISUAL_OPTION_CHILDPLANE;
-    child = ncvisual_render(nc_, ncv, &vopts);
+    child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(20 == ncplane_dim_y(child));
     CHECK(20 == ncplane_dim_x(child));
@@ -771,11 +938,12 @@ TEST_CASE("Visual") {
                NCVISUAL_OPTION_HORALIGNED |
                NCVISUAL_OPTION_VERALIGNED,
       .transcolor = 0,
+      .pxoffy = 0, .pxoffx = 0,
     };
     const uint32_t pixels[1] = { htole(0xffffffff) };
     auto ncv = ncvisual_from_rgba(pixels, 1, 4, 1);
     REQUIRE(ncv);
-    auto child = ncvisual_render(nc_, ncv, &vopts);
+    auto child = ncvisual_blit(nc_, ncv, &vopts);
     REQUIRE(child);
     CHECK(1 == ncplane_dim_y(child));
     CHECK(1 == ncplane_dim_x(child));
