@@ -71,19 +71,21 @@ echo "Cut $VERSION, signed to $TARBALL.asc"
 echo "Now uploadling the sig to https://github.com/dankamongmen/notcurses/releases"
 echo "The bastards are trying to immanentize the Eschaton"
 
+# restrict to files beginning with n* to leave out shared objects
+cd "$BUILDDIR" && find . -name n\*1 -maxdepth 0 -o -name t\*1 -o -name n\*3 -o -name \*.html \
+  | xargs tar czvf ../notcurses-doc-$VERSION.tar.gz --strip-components=1
+cd -
 # requires token in ~/.netrc
-github-release dankamongmen/notcurses create v$VERSION --name "v$VERSION—$QUIP" --publish $TARBALL.asc
-rm $TARBALL.asc
+gh release create v$VERSION --title "v$VERSION—$QUIP" $TARBALL.asc notcurses-doc-$VERSION.tar.gz
+rm $TARBALL.asc notcurses-doc-$VERSION.tar.gz
 
+# install the build for python, then uninstall it. gross; we ought do better FIXME.
 cd "$BUILDDIR"
 sudo make install
-# restrict to files beginning with n* to leave out shared objects
-tar czvf notcurses-doc-$VERSION.tar.gz n*.1 t*.1 n*.3 *.html
-github-asset dankamongmen/notcurses upload v$VERSION notcurses-doc-$VERSION.tar.gz
 cd ../cffi
 python3 setup.py sdist
 python3 setup.py build
-twine upload -s -udankamongmen dist/*
-
-cd "../$BUILDDIR"
+# requires an apitoken in ~/.pypirc
+twine upload -s dist/*
+cd -
 cat install_manifest.txt | sudo xargs rm
